#!/usr/bin/env python3
import sys, click
from toolbox.logger import Log
from .iam import iam
from .compartment import compartment
from .oss import oss
from .dbs import dbs
from .vault import vault
from .regions import regions
from .compute import compute
from .ocicli_wrapper import cli
from .oci_config import OCIconfig
from toolbox import misc
from toolbox.misc import debug
from .iam import get_latest_profile, get_latest_region
from .iam_nongc import update_latest_profile, force_cache, cache_all_hack
from toolbox.click_complete import complete_oci_regions, complete_oci_profiles

MESSAGE="OCI CLI Client" + misc.MOVE + "Current Profile: " + misc.GREEN + misc.UNDERLINE + get_latest_profile().upper() + misc.RESET + " Region: " + misc.GREEN + misc.UNDERLINE + get_latest_region(get_latest_profile()).upper() + misc.RESET

@click.group('oci', help=MESSAGE, context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()}, invoke_without_command=True)
@click.option('-p', '--profile', 'profile_name', help='profile name to use when working with ocitools', required=False, default=get_latest_profile(), shell_complete=complete_oci_profiles)
@click.option('-r', '--region', 'region', help='region name to use when working with ocitools', required=False, type=str, shell_complete=complete_oci_regions, default=None)
@click.option('-c', '--cache', help="refresh all cache for region and tenant", is_flag=True, required=False, default=False, show_default=True)
@click.pass_context
def CLI(ctx, profile_name, region, cache):
    ctx.ensure_object(dict)
    ctx.obj['PROFILE'] = profile_name
    ctx.obj['REGION'] = region
    if cache is True:
        force_cache(profile_name, region)
        sys.exit()
    if region != get_latest_region(profile_name) and region != None:
        CONFIG = OCIconfig()
        CONFIG.update_oci_config('config', profile_name, 'region', region)
        cache_all_hack(profile_name, region)
    if profile_name != get_latest_profile():
        update_latest_profile(profile_name)
        cache_all_hack(profile_name, region)
    Log.setup('ocitools', int(debug))
    pass
    
CLI.add_command(cli)
CLI.add_command(compute)
CLI.add_command(compartment)
CLI.add_command(dbs)
CLI.add_command(iam)
CLI.add_command(oss)
CLI.add_command(regions)
CLI.add_command(vault)

if __name__ == "__main__":
    cli(ctx)



import click, os, json
from toolbox.logger import Log
from .regionsclient import REGIONSclient
from tabulate import tabulate
from configstore.configstore import Config
from toolbox.misc import set_terminal_width
from .iam import get_latest_profile
from toolbox.menumaker import Menu

CONFIG = Config('ocitools')

@click.group('regions', invoke_without_command=True, help='module to manage region subscriptions', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.option('-m', '--menu', help='use the menu to perform region actions', is_flag=True, show_default=True, default=False, required=False)
@click.pass_context
def regions(ctx, menu):
    profile_name = ctx.obj['PROFILE']
    if menu:
        ctx.obj['MENU'] = True
    else:
        ctx.obj['MENU'] = False
    pass

@regions.command(help='manually refresh regions cached data', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
def refresh(ctx):
    profile_name = ctx.obj['PROFILE']
    _refresh('cached_regions', profile_name)

def _refresh(cache_type, profile_name):
    try:
        if cache_type == 'cached_regions':
            REGIONS = get_REGIONclient(profile_name, auto_refresh=False)
            RESULT = REGIONS.refresh('cached_regions', profile_name)
        return True
    except:
        False
    
@regions.command(help='show the data stored in cached regions', context_settings={'help_option_names':['-h','--help']})
@click.argument('region', required=False)
@click.pass_context
def show(ctx, region):
    profile_name = ctx.obj['PROFILE']
    _show(ctx, profile_name, region)

def _show(ctx, profile_name, region):
    if not region:
        if ctx.obj["MENU"]:
            DATA = []
            REGIONS = get_REGIONclient(profile_name, auto_refresh=False, cache_only=True)
            RESPONSE = REGIONS.get_cached_regions(profile_name)
            for data in RESPONSE:
                ID = data
                DATA.append(ID)
            INPUT = 'Region Manager'
            CHOICE = runMenu(DATA, INPUT)
            try:
                CHOICE = ''.join(CHOICE) 
            except:
                Log.critical("please select a region identifier to continue...")
            RESPONSE, DATA = REGIONS.describe(profile_name)
            print(RESPONSE, DATA)
        else:
            REGIONS = get_REGIONclient(profile_name, auto_refresh=False, cache_only=True)
            RESPONSE = REGIONS.get_cached_regions(profile_name)
            show_as_table(RESPONSE)
    else:
        REGIONS = get_REGIONclient(profile_name, auto_refresh=False, cache_only=True)
        RESPONSE = REGIONS.describe(region)
        Log.info(f"describing {region}:\n" + json.dumps(RESPONSE, indent=2, sort_keys=True, default=str))

def show_as_table(source_data):
    DATADICT = {}
    DATA = []
    for I in source_data:
        if 'last_cache_update' not in I:
            DATA.append(source_data[I])
            if DATA:
                DATADICT = DATA
    if DATADICT != []:
        Log.info(f"\n{tabulate(DATADICT, headers='keys', tablefmt='rst')}\n")

def get_REGIONclient(profile_name, region='us-ashburn-1', auto_refresh=False, cache_only=False):
    CLIENT = REGIONSclient(profile_name, region, cache_only)
    if auto_refresh:
        CLIENT.auto_refresh(profile_name)
    return CLIENT

def runMenu(DATA, INPUT):
    COUNT = 0
    FINAL = []
    TITLE = f'Region Menu: {INPUT}'
    for data in DATA:
        COUNT = COUNT + 1
        RESULTS = []
        RESULTS.append(data)
        FINAL.append(RESULTS)
    SUBTITLE = f'showing {COUNT} available object(s)'
    JOINER = '\t\t'
    FINAL_MENU = Menu(FINAL, TITLE, JOINER, SUBTITLE)
    CHOICE = FINAL_MENU.display()
    return CHOICE



#!/usr/bin/env python3
# Oracle OCI - Instance report script
# Version: 1.8 22-November 2018
# Written by: richard@oc-blog.com
# More info see: www.oc-blog.com
#
# This script will create a CSV report for all compute and DB instances (including ADW and ATP)
# in your OCI account, including predefined tags
#
# Instructions:
# - you need the OCI python API, this can be installed by running: pip install oci
# - you need the OCI CLI, this can be installed by running: pip install oci-cli
# - Make sure you have a user with an API key setup in the OCI Identity
# - Create a config file using the oci cli tool by running: oci setup config
# - In the script specify the config file to be used for running the report
# - You can specify any region in the config file, the script will query all enabled regions

import sys, os

import oci
import json
import shapes
import logging

AllPredefinedTags = True        # use only predefined tags from root compartment or include all compartment tags as well
NoValueString = "n/a"           # what value should be used when no data is available
FieldSeperator = ","            # what value should be used as field seperator
ReportFile = "/opt/native/gen2cli/logs/report.csv"
ReportJson = "/opt/native/gen2cli/logs/report.json"
EndLine = "\n"
home = ''

if len(sys.argv) != 4:
    print("INFO: all parameters are required to run this tool:")
    print("- 1st param = profile")
    print("- 2nd param = tenancy ocid")
    print("- 3rd param = compartment name")
    sys.exit()

profile = sys.argv[1]
tenantOcid = sys.argv[2]
myCompartment = sys.argv[3]
config = oci.config.from_file(profile_name=profile)

def generate_signer_from_config(config):
    signer = oci.signer.Signer(
        tenancy=config["tenancy"],
        user=config["user"],
        fingerprint=config["fingerprint"],
        private_key_file_location=config.get("key_file")
    )
    return signer

signer = generate_signer_from_config(config)
identity_client = oci.identity.IdentityClient(config, signer=signer)
compute_client = oci.core.ComputeClient(config, signer=signer)
virtual_network_client = oci.core.VirtualNetworkClient(config, signer=signer)
original_stdout = sys.stdout

logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)

def DisplayInstances(instances, compartmentName, instancetype, regionname):
  for instance in instances:
    sys.stdout = jsonfile
    print(instance)
    sys.stdout = original_stdout
    privateips = ""
    publicips = ""
    instancetypename = ""
    tagtxt = ""
    OS = ""

    if instancetype == "Compute":
      OCPU, MEM, SSD = shapes.ComputeShape(instance.shape)
      response = ComputeClient.list_vnic_attachments(compartment_id = instance.compartment_id, instance_id = instance.id)
      vnics = response.data
      try:
        for vnic in vnics:
          responsenic = NetworkClient.get_vnic(vnic_id=vnic.vnic_id)
          nicinfo = responsenic.data
          privateips = privateips + nicinfo.private_ip + " "
          publicips = publicips + nicinfo.public_ip + " "
      except:
        privateips = NoValueString
        publicips = NoValueString

      instancetypename = "Compute"
      namespaces = instance.defined_tags

      shape = instance.shape

      # Get OS Details
      try:
        response = ComputeClient.get_image(instance.source_details.image_id)
        imagedetails = response.data
        OS = imagedetails.display_name
      except:
        OS = NoValueString

      prefix,AD = instance.availability_domain.split(":")

    # Handle details for Database Instances
    if instancetype=="DB":
      OCPU, MEM, SSD = shapes.ComputeShape(instance.shape)
      OCPU = instance.cpu_core_count # Overwrite Shape's CPU count, with DB enabled CPU count
      response = databaseClient.list_db_nodes(compartment_id = instance.compartment_id, db_system_id = instance.id)
      dbnodes = response.data
      try:
        for dbnode in dbnodes:
          responsenic = NetworkClient.get_vnic(vnic_id=dbnode.vnic_id)
          nicinfo = responsenic.data
          privateips = privateips + nicinfo.private_ip + " "
          publicips = publicips + nicinfo.public_ip + " "
      except:
          privateips = NoValueString
          publicips = NoValueString

      instancetypename= "DB " + instance.database_edition
      OS = "Oracle Linux 6.8"
      shape = instance.shape
      prefix,AD = instance.availability_domain.split(":")

    # Handle details for Autonomous Database (ATP)
    if instancetype == "ATP":
      OCPU = instance.cpu_core_count
      MEM = NoValueString
      SSD = instance.data_storage_size_in_tbs
      instancetypename = "ATP"
      OS = NoValueString
      shape = "ATP"
      AD = regionname.upper()
      privateips = NoValueString
      publicips = NoValueString

    # Handle details for Autonomous Database (ADW)
    if instancetype == "ADW":
      OCPU = instance.cpu_core_count
      MEM = NoValueString
      SSD = instance.data_storage_size_in_tbs
      instancetypename = "ADW"
      OS = NoValueString
      shape = "ADW"
      AD = regionname.upper()
      privateips = NoValueString
      publicips = NoValueString

    try:
      namespaces = instance.defined_tags
      for customertag in customertags:
        try:
           tagtxt = tagtxt + FieldSeperator + namespaces[customertag[0]][customertag[1]]
        except:
           tagtxt = tagtxt + FieldSeperator + NoValueString
    except:
      tagtxt = ""  # No Tags

    line =   "{}{}".format(      instance.display_name,    FieldSeperator)
    line = "{}{}{}".format(line, instance.lifecycle_state, FieldSeperator)
    line = "{}{}{}".format(line, instancetypename,         FieldSeperator)
    line = "{}{}{}".format(line, shape,                    FieldSeperator)
    line = "{}{}{}".format(line, compartmentName,          FieldSeperator)
    line = "{}{}".format(line, AD)
    report.write(line + EndLine)

sys.stdout = original_stdout
report = open(ReportFile,'w')
jsonfile = open(ReportJson,'w')
customertags = []
header = "NAME,STATE,SERVICE,SHAPE,COMPARTMENT,AD".replace(",", FieldSeperator)

identity = oci.identity.IdentityClient(config)
user = identity.get_user(config["user"]).data
RootCompartmentID = tenantOcid

print ("logged in as:\t\t {} @ {}".format(user.description, config["region"]))
print ("querying region: ", end='')

response = identity.list_region_subscriptions(config["tenancy"])
regions = response.data

for region in regions:
    if region.region_name == profile:
        if region.is_home_region:
            home = "Home region"
        print ("\t {} ({}) {}".format(region.region_name, region.status, home))
report.write(header + EndLine)

ComputeClient = oci.core.ComputeClient(config)
NetworkClient = oci.core.VirtualNetworkClient(config)

# Check instances for all the underlaying Compartments
response = oci.pagination.list_call_get_all_results(identity.list_compartments,RootCompartmentID,compartment_id_in_subtree=True)
compartments = response.data

# Insert (on top) the root compartment
RootCompartment = oci.identity.models.Compartment()
RootCompartment.id = RootCompartmentID
RootCompartment.name = "root"
RootCompartment.lifecycle_state = "ACTIVE"
compartments.insert(0, RootCompartment)

for compartment in compartments:
  if myCompartment == compartment.name:
    compartmentName = compartment.name
  else:
    continue
  if compartment.lifecycle_state == "ACTIVE":
    print ("processing compartment:  " + compartmentName)
    compartmentID = compartment.id
    try:
      response = oci.pagination.list_call_get_all_results(ComputeClient.list_instances,compartment_id=compartmentID)
      if len(response.data) > 0:
        DisplayInstances(response.data, compartmentName, "Compute", region.region_name)
    except:
      pass

    databaseClient = oci.database.DatabaseClient(config)
    try:
      response = oci.pagination.list_call_get_all_results(databaseClient.list_db_systems,compartment_id=compartmentID)
      if len(response.data) > 0:
        DisplayInstances(response.data, compartmentName, "DB", region.region_name)
    except:
      pass

    try:
      response = oci.pagination.list_call_get_all_results(databaseClient.list_autonomous_data_warehouses,compartment_id=compartmentID)
      if len(response.data) > 0:
        DisplayInstances(response.data, compartmentName, "ADW", region.region_name)
    except:
      pass

    try:
      response = oci.pagination.list_call_get_all_results(databaseClient.list_autonomous_databases,compartment_id=compartmentID)
      if len(response.data) > 0:
        DisplayInstances(response.data, compartmentName, "ATP", region.region_name)
    except:
      pass

report.close()
jsonfile.close()
report = '/opt/native/gen2cli/logs/report.csv'
cmd = 'tabulate --header -s, -f rst %s' %(report)
print()
os.system(cmd)
print()
print("INFO: complete JSON output:  " + ReportJson)
print("INFO: short summary output:  " + ReportFile)



import click, os, json
from toolbox.logger import Log
from .dbsclient import DBSclient
from tabulate import tabulate
from configstore.configstore import Config
from toolbox.misc import set_terminal_width
from .iam import get_latest_profile
from toolbox.menumaker import Menu

CONFIG = Config('ocitools')

@click.group('dbs', invoke_without_command=True, help='dbs module to manage the DBS instances and related configuration', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.option('-m', '--menu', help='use the menu to perform DBS actions', is_flag=True, show_default=True, default=False, required=False)
@click.pass_context
def dbs(ctx, menu):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    if menu:
        ctx.obj['MENU'] = True
    else:
        ctx.obj['MENU'] = False
    pass

@dbs.command(help='manually refresh dbs cached data', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
def refresh(ctx):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _refresh('cached_dbs_instances', oci_region, profile_name)

def _refresh(cache_type, oci_region, profile_name):
    try:
        if cache_type == 'cached_dbs_instances':
            DBS = get_DBSclient(profile_name, oci_region, auto_refresh=False)
            RESULT = DBS.refresh('cached_dbs_instances', profile_name)
        return True
    except:
        False
    
@dbs.command(help='show the data stored in dbs cache', context_settings={'help_option_names':['-h','--help']})
@click.argument('database', required=False)
@click.pass_context
def show(ctx, database):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _show(ctx, profile_name, oci_region, database)

def _show(ctx, profile_name, oci_region, database):
    if not database:
        if ctx.obj["MENU"]:
            DATA = []
            DBS = get_DBSclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
            RESPONSE = DBS.show_dbs_instances(profile_name)
            for data in RESPONSE[oci_region]:
                ID = data
                DATA.append(ID)
            INPUT = 'DBS Manager'
            CHOICE = runMenu(DATA, INPUT)
            try:
                CHOICE = ''.join(CHOICE) 
            except:
                Log.critical("please select a DBS Identifier to continue...")
            RESPONSE = DBS.describe(CHOICE)
            Log.info(f"describing {CHOICE}:\n" + json.dumps(RESPONSE, indent=2, sort_keys=True, default=str))
        else:
            DBS = get_DBSclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
            RESPONSE = DBS.show_dbs_instances(profile_name)
            try:
                show_as_table(RESPONSE[oci_region], name='DBS TABLE')
            except:
                Log.warn(f'there are 0 DBS instances in {oci_region} and profile: {profile_name}')
                pass
    else:
        DBS = get_DBSclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
        RESPONSE = DBS.describe(database)
        Log.info(f"describing {database}:\n" + json.dumps(RESPONSE, indent=2, sort_keys=True, default=str))

def show_as_table(source_data, name=None):
    DATA = []
    for ENTRY in source_data:
        DATA_ENTRY = {}
        if ENTRY != 'last_cache_update':
            DATA_ENTRY['display_name'] = ENTRY
            for PROPERTY in source_data[ENTRY]:
                DATA_ENTRY[PROPERTY] = source_data[ENTRY][PROPERTY]
            DATA.append(DATA_ENTRY)
    if DATA != []:
        Log.info(f"{name}\n{tabulate(DATA, headers='keys', tablefmt='rst')}\n")

def get_DBSclient(profile_name, region='us-ashburn-1', auto_refresh=False, cache_only=False):
    CLIENT = DBSclient(profile_name, region, cache_only)
    if auto_refresh:
        CLIENT.auto_refresh(profile_name)
    return CLIENT

def runMenu(DATA, INPUT):
    COUNT = 0
    FINAL = []
    TITLE = f'DBS Menu: {INPUT}'
    for data in DATA:
        COUNT = COUNT + 1
        RESULTS = []
        RESULTS.append(data)
        FINAL.append(RESULTS)
    SUBTITLE = f'showing {COUNT} available object(s)'
    JOINER = '\t\t'
    FINAL_MENU = Menu(FINAL, TITLE, JOINER, SUBTITLE)
    CHOICE = FINAL_MENU.display()
    return CHOICE

def get_region(ctx, profile_name):
    OCI_REGION = ctx.obj['REGION']
    if not OCI_REGION:
        DBS = get_DBSclient(profile_name)
        OCI_REGION = DBS.get_region_from_profile(profile_name)
    return OCI_REGION



import random
import string

def get_random_password(length):
    random_source = string.ascii_letters + string.digits
    # select 1 lowercase
    password = random.choice(string.ascii_lowercase)
    # select 1 uppercase
    password += random.choice(string.ascii_uppercase)
    # select 1 digit
    password += random.choice(string.digits)

    # generate other characters
    for i in range(length):
        password += random.choice(random_source)

    password_list = list(password)
    # shuffle all characters
    random.SystemRandom().shuffle(password_list)
    password = ''.join(password_list)
    return password



#!/usr/bin/env python3
# ==============================================================================
# Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
# ==============================================================================

import sys, argparse, logging
import sys, os, threading, socket, argparse, re, tty, termios, fnmatch, os.path, pexpect, time, datetime, collections, locale, socket, struct, subprocess
import shutil
from collections import defaultdict
from termcolor import cprint
from time import sleep
from glob import glob
from multiprocessing import Process
from tqdm.auto import tqdm
import common
import color

import oci
from oci.object_storage.models import CreateBucketDetails

# initial directory stack
pushstack = list()
global targets, regions
targets = {}

home = os.getenv("HOME")
lock = threading.Lock()

# get our regions
with open(home + "/.oci/config") as f:
    regions = []
    for line in f:
        if line.startswith('['):
            s = line.replace(']','').replace('[','').strip()
            regions.append(s)
    regions.append("ALL")

class ProgressBar(object):

    DEFAULT = 'Progress: %(bar)s %(percent)3d%%'
    FULL = '%(bar)s %(current)d/%(total)d (%(percent)3d%%) %(remaining)d to go'

    def __init__(self, num_lines, width=80, fmt=DEFAULT, symbol='#',
                 output=sys.stderr):
        assert len(symbol) == 1

        total = num_lines
        self.total = total
        self.width = width
        self.symbol = symbol
        self.output = output
        self.fmt = re.sub(r'(?P<name>%\(.+?\))d',
            r'\g<name>%dd' % len(str(total)), fmt)
        self.current = 0

    def __call__(self):
        percent = self.current / float(self.total)
        size = int(self.width * percent)
        remaining = self.total - self.current
        bar = '[' + self.symbol * size + ' ' * (self.width - size) + ']'

        args = {
            'total': self.total,
            'bar': bar,
            'current': self.current,
            'percent': percent * 100,
            'remaining': remaining
        }
        print('\r' + self.fmt % args, file=self.output, end='')

    def done(self):
        self.current = self.total
        self()
        print('', file=self.output)

def delBucket(object_storage, namespace, bucket, profile):

    try:
        response = oci.pagination.list_call_get_all_results(object_storage.list_preauthenticated_requests, namespace_name=namespace, bucket_name=bucket)
    except:
        print("FAIL: bucket {} not found in region:  {}".format(bucket, profile), end='')
        print(common.MOVE2 + common.RED + "FAILURE" + common.RESET)
        return -1

    if response:
        for auth in response.data:
            oci.object_storage.ObjectStorageClient.delete_preauthenticated_request(object_storage, namespace_name=namespace, bucket_name=bucket, par_id=auth.id)

    try:
        object_storage.delete_bucket(namespace, bucket)
        print('INFO: deleted bucket ' + bucket + ' from: ' + profile)
        return 0
    except:
        cmd='oci --profile %s os object bulk-delete --bucket-name %s --force >/dev/null 2>&1' %(profile, bucket)
        os.system(cmd)
        # delete the bucket now
        object_storage.delete_bucket(namespace, bucket)
        print('INFO: deleted bucket ' + bucket + ' from: ' + profile)
        return 0
    return 0

def createBucket(object_storage, namespace, bucket, profile):
    if args.ocid:
        c = str(args.ocid)

    request = CreateBucketDetails(public_access_type='NoPublicAccess')     # private bucket
    request.compartment_id = c
    request.name = bucket
    bucket = object_storage.create_bucket(namespace, request)

    if bucket.data.etag:
        print("INFO: bucket created for " + profile + " and bucket tag " + bucket.data.etag)
        return 0

def upload_to_object_storage(config, namespace, bucket, path, profile):

    with open(path, "rb") as in_file:
        name = os.path.basename(path)
        ostorage = oci.object_storage.ObjectStorageClient(config)
        try:
            ostorage.put_object(namespace,
                                bucket,
                                name,
                                in_file)
            print("INFO: finished uploading {} to {}".format(name, profile), end='')
            print(common.MOVE2 + common.GREEN + "SUCCESS" + common.RESET)
        except:
            print("ERROR")

class BamBam(threading.Thread):

    output_lock = threading.Lock()

    def __init__(self):
        threading.Thread.__init__(self)
        # A flag to notify the thread that it should finish up and exit
        self.kill_received = False

    def run(self):

        while not self.kill_received:
            try:
                line = str(queue.get(True, 1))
                profile = line.strip()
            except Queue.Empty:
                continue

            chk = any(c.isalpha() for c in profile)

            if chk is False:
                break

            config = oci.config.from_file(profile_name=profile)
            object_storage = oci.object_storage.ObjectStorageClient(config)
            namespace = object_storage.get_namespace(compartment_id=request.compartment.id).data
            client = oci.identity.IdentityClient(config)
            request.compartment_id = c

            if args.create:
                try:
                    result = self.createBucket(object_storage, namespace, bucket, profile)
                except:
                    print(common.RED + "ERROR: " + common.RESET + " unable to create bucket")
                    result = -1
                    pass
            elif args.delete:
                try:
                    result = self.delBucket(object_storage, namespace, bucket, profile)
                except:
                    print(common.RED + "ERROR: " + common.RESET + " unable to delete bucket")
                    result = -1
                    pass
            elif args.fname:
                self.uploadObjects(object_storage, namespace, bucket, profile, config)
                result = 0
            elif args.verify:
                result = self.verifyProfile(config, client, profile)
            elif args.agents or args.patches or args.solr or args.cookbooks:
                prefix_files_name = 'patch'
                listfiles = object_storage.list_objects(namespace,bucket,prefix=prefix_files_name)

                if not listfiles.data.objects:
                    print("INFO: no files found to be deleted in profile " + profile)
                    result = 0
                else:
                    for filenames in listfiles.data.objects:
                        object_storage.delete_object(namespace, bucket,filenames.name)
                        print("INFO: deleted " + filenames.name + " from bucket " + bucket + " and profile " + profile)
                    result = 0

            return_queue.put([profile, result])
            continue

    def verifyProfile(self, config, client, profile):

        try:
            oci.config.validate_config(config)
            blah = client.list_regions()
            with self.output_lock:
                print("INFO: verifying profile and access for: " + profile, end='')
                print(common.MOVE2 + common.GREEN + "SUCCESS" + common.RESET)
            return 0
        except:
            with self.output_lock:
                print("INFO: verifying profile and access for: " + profile, end='')
                print(common.MOVE2 + common.RED + "FAILURE" + common.RESET)
            return -1
        return 0

    def delBucket(self, object_storage, namespace, bucket, profile):

        try:
            response = oci.pagination.list_call_get_all_results(object_storage.list_preauthenticated_requests, namespace_name=namespace, bucket_name=bucket)
        except:
            print("FAIL: bucket {} not found in region:  {}".format(bucket, profile), end='')
            print(common.MOVE2 + common.RED + "FAILURE" + common.RESET)
            return -1

        if response:
            for auth in response.data:
                oci.object_storage.ObjectStorageClient.delete_preauthenticated_request(object_storage, namespace_name=namespace, bucket_name=bucket, par_id=auth.id)

        try:
            object_storage.delete_bucket(namespace, bucket)
            print('INFO: deleted bucket from: ' + profile)
            return 0
        except:
            cmd='oci --profile %s os object bulk-delete --bucket-name %s --force >/dev/null 2>&1' %(profile, bucket)
            os.system(cmd)
            object_storage.delete_bucket(namespace, bucket)
            print('INFO: deleted bucket from: ' + profile)
            return 0
        return 0

    def createBucket(self, object_storage, namespace, bucket, profile):
        if args.ocid:
            c = str(args.ocid)

        #request = CreateBucketDetails(public_access_type='ObjectWrite')
        request = CreateBucketDetails(public_access_type='NoPublicAccess')
        request.compartment_id = c
        request.name = bucket
        bucket = object_storage.create_bucket(namespace, request)

        if bucket.data.etag:
            print("INFO: bucket created for " + profile + " and bucket tag " + bucket.data.etag)
            return 0

        return -1

    def upload_to_object_storage(self, config, namespace, bucket, path, profile):

        with open(path, "rb") as in_file:
            name = os.path.basename(path)
            ostorage = oci.object_storage.ObjectStorageClient(config)
            try:
                ostorage.put_object(namespace,
                                    bucket,
                                    name,
                                    in_file)
                print("INFO: finished uploading {} to {}".format(name, profile), end='')
                print(common.MOVE2 + common.GREEN + "SUCCESS" + common.RESET)
            except:
                print("ERROR")

    def uploadObjects(self, object_storage, namespace, bucket, profile, config):

        object_name = str(args.fname)
        if not os.path.isfile(object_name) and not os.path.isdir(object_name):
            print(common.RED + "ERROR: " + common.RESET + " unable to upload object to OSS because file does not exist " + object_name)
            parser.parse_args(['-h'])

        if os.path.isfile(object_name):
            with open(object_name, 'rb') as f:
                try:
                    obj = object_storage.put_object(namespace, bucket, object_name, f)
                    print("INFO: finished uploading {} to {}".format(object_name, profile), end='')
                    print(common.MOVE2 + common.GREEN + "SUCCESS" + common.RESET)
                except:
                    print(common.RED + "ERROR: " + common.RESET + " unable to upload object to OSS " + object_name)
                    pass
        else:
            dir = args.fname + os.path.sep + "*"
            proc_list = []

            for f in glob(dir):
                if os.path.isfile(f):
                    p = Process(target=upload_to_object_storage, args=(config, namespace, bucket, f, profile))
                    p.start()
                    proc_list.append(p)

            for job in proc_list:
                job.join()

        return 0

def startWork(profiles):

    global queue
    global return_queue
    global interval
    global number

    queue = Queue.Queue(0)
    return_queue = Queue.Queue(0)

    number = 0
    threadlist = []
    start = time.time()
    num_of_threads = maxthread

    if len(targets) < maxthread:
        num_of_threads = len(targets)

    for x in range(num_of_threads):
        t = BamBam()
        threadlist.append(t)
        t.start()

    for p in targets:
        queue.put(p)

    for t in threadlist:
        queue.put([0, 0])

    while has_live_threads(threadlist):
        try:
            [t.join(1) for t in threadlist
             if t is not None and t.isAlive()]
        except KeyboardInterrupt:
            print(common.RED + "\nCaught KeyboardInterrupt, exiting...\n" + common.RESET)
            for t in threadlist:
                t.kill_received = True
            return_code = 1
            try:
                sys.exit(0)
            except SystemExit:
                os._exit(0)

    for x in range(len(targets)):
        profile, results = return_queue.get()

    finish = time.time()
    interval = finish - start

def timeMessage():

    print (common.YELLOW + "#####################################################################" + common.RESET)
    print (common.YELLOW + "# EXECUTION TIME: " + str(interval) + " seconds" + common.RESET)
    print (common.YELLOW + "#####################################################################\n" + common.RESET)

parser = argparse.ArgumentParser(description=common.BLUE + 'OSS utility' + common.RESET)
commands = parser.add_argument_group('available tasks')
commands.add_argument('--bucket', default=False, type=str, dest='bucket', help=common.BLUE + 'provide a bucket name for operations'  + common.RESET)
commands.add_argument('--delete', action='store_true', dest='delete', default=False, help=common.BLUE + 'delete a bucket in compartment'  + common.RESET)
commands.add_argument('--create', action='store_true', dest='create', default=False, help=common.BLUE + 'create a bucket in compartment'  + common.RESET)
commands.add_argument('--verify', action='store_true', dest='verify', default=False, help=common.BLUE + 'verify profile connection keys'  + common.RESET)
commands.add_argument('--file', dest='fname', default=False, type=str, help=common.BLUE + 'provide file/directory name to upload to specific bucket'  + common.RESET)
commands.add_argument('--pull', dest='object', default=False, type=str, help=common.BLUE + 'provide object name to download from bucket'  + common.RESET)

options = parser.add_argument_group('options')
options.add_argument('--profile', default=False, nargs='?', help=common.BLUE + 'supply profile name for connection' + common.RESET, choices=regions)
options.add_argument('--compartment', default=False, type=str, dest="ocid", help=common.BLUE + 'supply optional compartment OCID' + common.RESET)

def has_live_threads(threadlist):
    return True in [t.isAlive() for t in threadlist]

def populateTargets(fname):

    num_lines = sum(1 for line in open(fname))
    progress = ProgressBar(num_lines, fmt=ProgressBar.FULL)
    cprint("\n[*] compiling profile target list ", "green")

    my_file = open(fname, 'r')
    number = 0
    flag = False

    for line in my_file:
        progress.current += 1
        progress()
        number = number + 1
        head, sep, tail = line.partition(',')
        profile = head.strip()

        flag = True

        targets[profile] = [number]
        targets[profile].append(profile)
    progress.done()
    progress.current = 0

    return targets

def main():

    start = time.time()
    global maxthread, args, bucket

    maxthread = 40
    count_args = len(sys.argv)

    if count_args <= 0:
        parser.parse_args(['-h'])
    else:
        args = parser.parse_args()

    if args.profile:
        profile = str(args.profile)

        if args.bucket:
            bucket = str(args.bucket)
        elif not args.bucket and args.verify:
            verify = True
        else:
            parser.parse_args(['-h'])

        if profile == 'ALL':
            populateTargets(regions)
            try:
                startWork(regions)
            except KeyboardInterrupt:
                print(common.RED + "\nCaught KeyboardInterrupt, exiting...\n" + common.RESET)
                return_code = 1
                try:
                    sys.exit(0)
                except SystemExit:
                    os._exit(0)
        else:
            c = str(args.ocid)
            config = oci.config.from_file(profile_name=profile)
            object_storage = oci.object_storage.ObjectStorageClient(config)
            namespace = object_storage.get_namespace(compartment_id=c).data
            client = oci.identity.IdentityClient(config)

            if args.verify:
                print("INFO: verifying profile and access for: " + profile, end='')
                try:
                    oci.config.validate_config(config)
                    blah = client.list_regions()
                    print(common.MOVE2 + common.GREEN + "SUCCESS" + common.RESET)
                except:
                    print(common.MOVE2 + common.RED + "FAILURE" + common.RESET)
                    sys.exit()

            elif args.create:
                try:
                    createBucket(object_storage, namespace, bucket, profile)
                except:
                    print(common.RED + "ERROR: " + common.RESET + " unable to create bucket")
                    pass
            elif args.delete:
                try:
                    delBucket(object_storage, namespace, bucket, profile)
                except:
                    print(common.RED + "ERROR: " + common.RESET + " unable to delete bucket")
                    pass
            elif args.object:
                object_name = str(args.object)
                print('INFO: retrieving file from object storage')
                get_obj = object_storage.get_object(namespace, bucket, object_name)
                total = int(get_obj.headers.get('content-length', 0))
                output = home + '/.gen2/downloads/%s' %(str(object_name))
                os.makedirs(os.path.dirname(output), exist_ok=True)
                print()
                try:
                    with open(output, 'wb', buffering = 2 ** 24) as f, tqdm(
                        desc=str(object_name),
                        total=total,
                        unit='iB',
                        unit_scale=True,
                        unit_divisor=1024,
                    ) as bar:
                        for data in get_obj.data.iter_content(chunk_size=1024):
                            size = f.write(data)
                            bar.update(size)
                except KeyboardInterrupt:
                    print(color.RED + "\nCaught KeyboardInterrupt, exiting...\n" + color.RESET)
                    try:
                        sys.exit(0)
                    except SystemExit:
                        os._exit(0)
                print()
            elif args.fname:
                object_name = str(args.fname).rstrip("/")

                if not os.path.isfile(object_name) and not os.path.isdir(object_name):
                    print(common.RED + "ERROR: " + common.RESET + " unable to upload object to OSS because file does not exist " + object_name)
                    parser.parse_args(['-h'])

                if os.path.isfile(object_name):
                    with open(object_name, 'rb') as f:
                        try:
                            obj = object_storage.put_object(namespace, bucket, object_name, f)
                            print("INFO: finished uploading {} to {}".format(object_name, profile), end='')
                            print(common.MOVE2 + common.GREEN + "SUCCESS" + common.RESET)
                        except:
                            print(common.RED + "ERROR: " + common.RESET + " unable to upload object to OSS " + object_name)
                            pass
                else:
                    dir = args.fname + os.path.sep + "*"
                    proc_list = []

                    for f in glob(dir):
                        if os.path.isfile(f):
                            print("INFO: starting upload for {}".format(f))
                            p = Process(target=upload_to_object_storage, args=(config, namespace, bucket, f, profile))
                            p.start()
                            proc_list.append(p)

                    print()
                    for job in proc_list:
                        job.join()
                    print()
    else:
        print(common.RED + "ERROR: " + common.RESET + " please provide a profile for operations\n")
        parser.parse_args(['-h'])

if __name__ == "__main__":

    try:
        main()
    except KeyboardInterrupt:
        print(common.RED + "\nCaught KeyboardInterrupt, exiting...\n" + common.RESET)
        return_code = 1
        try:
            sys.exit(0)
        except SystemExit:
            os._exit(0)



from toolbox.logger import Log
from toolbox.misc import detect_environment
from configstore.configstore import Config
from .oci_config import OCIconfig
from . import iam_nongc
import datetime
from tabulate import tabulate
import os, oci, json

CONFIGSTORE = Config('ocitools')

class VAULTclient():

    def __init__(self, profile_name, region='us-ashburn-1', cache_only=False):
        self.CONFIGSTORE = Config('ocitools')
        self.CONFIG = OCIconfig()
        self.CACHE_UPDATE_INTERVAL = 60*60*24*7 # update cache every week
        self.OCI_PROFILE = profile_name
        self.OCI_REGION = region
        self.CACHE_ONLY = cache_only

        # get these from oci.config based on profile name and validate the config
        self.config = oci.config.from_file("~/.oci/config", profile_name)
        self.OCID = self.tenant = self.config.get("tenancy")
        self.user = self.config.get("user")
        self.fingerprint = self.config.get("fingerprint")
        self.keyfile = self.config.get("key_file")
        self.CONFIG_FROM_FILE = {
                'tenancy': self.tenant,
                'user': self.user,
                'fingerprint': self.fingerprint,
                'key_file': self.keyfile,
                'region': self.OCI_REGION
        }
        try:
            oci.config.validate_config(self.CONFIG_FROM_FILE)
        except:
            Log.critical('unable to setup a proper oci configuration file')

        if not self.CACHE_ONLY:
            self.get_connections()

    def get_connections(self):
        #Log.info(f"connecting to OCI as {self.OCI_PROFILE} via {self.OCI_REGION}...")
        self.CLIENT = oci.identity.IdentityClient(self.CONFIG_FROM_FILE)
        self.TENANTNAME = self.CLIENT.get_tenancy(tenancy_id=self.OCID).data.name
        self.VAULT = oci.vault.VaultsClient(self.CONFIG_FROM_FILE)
        self.SECRETS = oci.secrets.SecretsClient(self.CONFIG_FROM_FILE)
        self.KMS_VAULT = oci.key_management.KmsVaultClient(self.CONFIG_FROM_FILE)
        self.COMPOSITE = oci.key_management.KmsVaultClientCompositeOperations(self.KMS_VAULT)
        return self.CLIENT, self.TENANTNAME, self.VAULT, self.SECRETS, self.KMS_VAULT, self.COMPOSITE

    def get_vaults_cache(self, profile_name='DEFAULT'):
        VAULTS_CACHE = {}
        if profile_name not in self.CONFIGSTORE.PROFILES:
            self.CONFIGSTORE.create_profile(profile_name)
        TIMESTAMP = str(datetime.datetime.now().timestamp())
        self.get_connections()
        Log.info("caching vault data across all compartments...")
        COMPARTMENTS = self.get_compartments()
        for COMPARTMENT in COMPARTMENTS:
            VAULTS = []
            try:
                VAULTS = self.KMS_VAULT.list_vaults(compartment_id=COMPARTMENT.id).data
                COMP_NAME = str(COMPARTMENT.name)
                COMP_OCID = str(COMPARTMENT.id)
                if len(VAULTS) > 0:
                    Log.debug("compartment name: " + COMP_NAME + " holds vaults in region " + self.OCI_REGION)
                    for DATA in VAULTS:
                        if DATA:
                            ID = DATA.id
                            NAME = DATA.display_name
                            CRYPTO_ENDPOINT = DATA.crypto_endpoint
                            MGMT_ENDPOINT = DATA.management_endpoint
                            STATE = DATA.lifecycle_state
                            VAULTS_CACHE[ID] = {
                                'display_name': NAME,
                                'compartment': COMP_NAME,
                                'crypto_endpoint': CRYPTO_ENDPOINT,
                                'management_endpoint': MGMT_ENDPOINT,
                                'lifecycle_state': STATE
                            }
            except oci.exceptions.ServiceError as e:
                VAULTS = ''
                pass
        VAULTS_CACHE['last_cache_update'] = TIMESTAMP
        DICT = {}
        DICT[self.OCI_REGION] = VAULTS_CACHE
        self.CONFIGSTORE.update_metadata(DICT, 'cached_vaults', profile_name, True)

    def describe(self, vault_ocid, profile_name):
        try:
            self.get_connections()
        except:
            return None
        RESPONSE = self.KMS_VAULT.get_vault(vault_ocid).data
        DICT = {}
        DICT[vault_ocid] = RESPONSE
        return DICT

    def get_cached_vaults(self, profile_name):
        if profile_name in self.CONFIGSTORE.PROFILES:
            if 'cached_vaults' in self.CONFIGSTORE.PROFILES[profile_name]['metadata']:
                return self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_vaults'][self.OCI_REGION]
        else:
            return None

    def refresh(self, type, profile_name):
        if type == 'cached_vaults':
            self.get_vaults_cache(profile_name)
        self.CONFIGSTORE = Config('ocitools')

    def get_compartments(self):
        try:
            self.get_connections()
        except:
            return None
        COMPARTMENTS = self.CLIENT.list_compartments(self.OCID, compartment_id_in_subtree=True).data
        ROOT_COMPARTMENT = self.CLIENT.get_compartment(compartment_id=self.OCID).data
        COMPARTMENTS.append(ROOT_COMPARTMENT)
        return COMPARTMENTS

    def create_vault(self, comp_id, vault_name):
        DETAILS = oci.key_management.models.CreateVaultDetails(compartment_id=comp_id, vault_type="DEFAULT", display_name=vault_name)
        try:
            RESPONSE = self.COMPOSITE.create_vault_and_wait_for_state(DETAILS, wait_for_states=[oci.key_management.models.Vault.LIFECYCLE_STATE_ACTIVE])
        except Exception as err:
            Log.critical(err)
        return RESPONSE

    def delete_vault(self, vault_id, deletion_time):
        schedule_vault_deletion_details = oci.key_management.models.ScheduleVaultDeletionDetails(time_of_deletion=deletion_time)
        RESPONSE = self.COMPOSITE.schedule_vault_deletion_and_wait_for_state(vault_id, schedule_vault_deletion_details=schedule_vault_deletion_details, wait_for_states=[oci.key_management.models.Vault.LIFECYCLE_STATE_PENDING_DELETION])
        return RESPONSE

    def get_region_from_profile(self, profile_name):
        REGION = self.CONFIG.get_from_config('config', 'region', profile_name=profile_name)
        if REGION is None:
            Log.critical("Please run goat oci iam authenticate for the target profile before using the oss module")
        return REGION

    def get_vaults(self, comp_id):
        VAULTS = self.KMS_VAULT.list_vaults(compartment_id=comp_id, sort_by='DISPLAYNAME', sort_order='ASC').data
        return VAULTS

    def auto_refresh(self, profile_name='DEFAULT'):
        TIME_NOW = datetime.datetime.now().timestamp()
        try:
            VAULT_TS = self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_vaults']['last_cache_update']
        except KeyError:
            VAULT_TS = 0
        if TIME_NOW - float(VAULT_TS) > self.CACHE_UPDATE_INTERVAL:
            self.get_vaults_cache(profile_name)
        self.CONFIGSTORE = Config('ocitools')



# ==============================================================================
# Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
# ==============================================================================
# common variables and functions for python can be imported from this file

from contextlib import contextmanager
from contextlib import closing
from subprocess import Popen, PIPE

import socket
import color

import sys, os, pickle, re, socket, subprocess

RESET = '\033[0m'
CYAN = '\033[0;36m'
RED = '\033[1;31m'
YELLOW = '\033[0;33m'
GREEN = '\033[0;32m'
MAGENTA = '\033[0;35m'
BLUE = '\033[38;5;69m'
ORANGE = '\033[38;5;172m'
UNDERLINE = '\033[4m'
BOLD = '\033[1m'
SCREEN_WIDTH = 95
MOVE1 = '\033[70G'
MOVE2 = '\033[100G'
myhost = socket.gethostname().lower()
site = myhost.split("-")[0].strip().upper()
#gatewayfile = '/opt/ORCLemaas/omcdevops/omccli/scratch/gateways.csv'
#gateways = [line.split(',')[1].strip().lower() for line in open(gatewayfile)]
#secondary_gw = [line.split(',')[2].strip().lower() for line in open(gatewayfile)]
#draw_layout = '/opt/ORCLemaas/omcdevops/omccli/modules/omcshell/lib/OmcUi.py'
#draw_usage = '/opt/ORCLemaas/omcdevops/omccli/modules/omcshell/lib/Usage.py'

def printBanner():

    if 'US2Z24' in site or 'NL1Z15' in site:
        color.printTitle()
    else:
        os.system(draw_layout)

def check_socket(host, port):
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
        if sock.connect_ex((host, port)) == 0:
            return 0
        else:
            return -1

def getGateway():
    for s in gateways:
        if site.lower() in s:
            gateway = str(s)
            gateway = ''.join(gateway)
            break
    return gateway

def getSecondary():
    for s in secondary_gw:
        if site.lower() in s:
            secondary = str(s)
            secondary = ''.join(secondary)
            break
    return secondary

def get_username():
    from os import environ, getcwd
    user = (lambda: environ["USERNAME"] if "C:" in getcwd() else environ["USER"])()
    return user

def printInfo(message):

    mydate = os.popen("date +'%A %Y-%m-%d %R %Z'").read().strip()
    message = GREEN + "[ MESSAGE ]" + RESET + " [ %s ] [ %s ]" %(mydate, message)
    print(message)

def printWarn(message):

    mydate = os.popen("date +'%A %Y-%m-%d %R %Z'").read().strip()
    message = YELLOW + "[ WARNING ]" + RESET + " [ %s ] [ %s ]" %(mydate, message)
    print(message)

def printFail(message):

    mydate = os.popen("date +'%A %Y-%m-%d %R %Z'").read().strip()
    message = RED + "[ FAILURE ]" + RESET + " [ %s ] [ %s ]" %(mydate, message)
    print(message)

def info(string,a=''):
    if a != '':
        print(YELLOW + "INFO: " + RESET + string%a)
    else:
        print(YELLOW + "INFO: " + RESET + string)

def error(string,a=''):
    if a != '':
        print(RED + "ERROR:" + RESET + string%a)
    else:
        print(RED + "ERROR:" + RESET + string)

def warn(string,a=''):
    if a != '':
        print(YELLOW + "WARN: " + RESET + string%a)
    else:
        print(YELLOW + "WARN: " + RESET + string)

def getTier():
    if any(re.findall(r'agt-1|agt-2|gwaytier-service-mgr|serviceregistrynode-1', myhost, re.IGNORECASE)):
        tier = 'GW'
    elif any(re.findall(r'ctr-1|service-manager-and-provisioning1|service-mgr-prov1|smandprovisioningnode', myhost, re.IGNORECASE)):
        tier = 'CT'
    else:
        tier = 'UNKNOWN'
    return tier

def startTunnel(gateway):

    # run a connection to gateway node in the background, make things faster for future runs on this node
    # tear it down later
    cmd = 'ssh -F /opt/ORCLemaas/omcdevops/omccli/.ssh/config -o ConnectTimeout=30 -tt -N %s >/dev/null 2>&1 &' %(str(gateway))
    os.system(cmd)
    return

def connectGateway(gateway, start_shell):

    if start_shell is True:
        cmd = "ssh -q -t " + str(gateway) + " \"omc\"" + " -o ""ProxyCommand=nc -x localhost:54021 %h %p"""
    else:
        cmd = "ssh -q %s" %(gateway)
    os.system(cmd)
    return

def sshStatus(gateway):

    cmd = 'ps -ef | grep ssh| grep %s | grep -v grep | awk "{print \$2}"' %(gateway)
    proc = subprocess.Popen([cmd], stdout=subprocess.PIPE, shell=True)
    (out, err) = proc.communicate()

    return out, err

def stopTunnel(pid):

    cmd = 'kill -9 %s >/dev/null 2>&1' %(pid)
    os.system(cmd)

def tunnelTasks(action):

    sockit = {}
    number = 0

    # start up our tunnel immediately
    gateway = getGateway()
    secondary = getSecondary()
    h = '127.0.0.1'
    port1 = 54021
    res = check_socket(h, port1)
    port2 = 54022
    ret = check_socket(h, port2)

    if res != 0:
        # start tunnel in background
        if action == 'start':
            startTunnel(gateway)
    else:
        # get pid
        pid = sshStatus(gateway)[0].strip()
        if action == 'start':
            if not pid:
                startTunnel(gateway)
        elif action == 'stop':
            if pid:
                stopTunnel(pid)
        sockit[gateway] = [number]
        sockit[gateway].append(pid)
        sockit[gateway].append(port1)
        number = number + 1

    if ret != 0:
        # start tunnel in background
        if action == 'start':
            startTunnel(secondary)
    else:
        # get pid
        pid = sshStatus(secondary)[0].strip()
        if action == 'start':
            if not pid:
                startTunnel(secondary)
        elif action == 'stop':
            if pid:
                stopTunnel(pid)
        sockit[secondary] = [number]
        sockit[secondary].append(pid)
        sockit[secondary].append(port2)
        number = number + 1

    return sockit



from toolbox.logger import Log
from toolbox.misc import detect_environment
from configstore.configstore import Config
from .oci_config import OCIconfig
from . import iam_nongc
import datetime
from tabulate import tabulate
import os, oci, json
from . import shapes

CONFIGSTORE = Config('ocitools')

class REGIONSclient():

    def __init__(self, profile_name, region='us-ashburn-1', cache_only=False):
        self.CONFIGSTORE = Config('ocitools')
        self.CONFIG = OCIconfig()
        self.CONFIG_FROM_FILE = oci.config.from_file(profile_name=profile_name)
        self.CACHE_UPDATE_INTERVAL = 60*60*24*7 # update cache every week
        self.OCI_PROFILE = profile_name
        self.OCI_REGION = region
        self.CACHE_ONLY = cache_only

        # get these from oci.config based on profile name and validate the config
        self.config = oci.config.from_file("~/.oci/config", profile_name)
        self.OCID = self.tenant = self.config.get("tenancy")
        self.user = self.config.get("user")
        self.fingerprint = self.config.get("fingerprint")
        self.keyfile = self.config.get("key_file")
        self.CONFIG_FROM_FILE = {
                'tenancy': self.tenant,
                'user': self.user,
                'fingerprint': self.fingerprint,
                'key_file': self.keyfile,
                'region': self.OCI_REGION
        }
        try:
            oci.config.validate_config(self.CONFIG_FROM_FILE)
        except:
            Log.critical('unable to setup a proper oci configuration file')

        if not self.CACHE_ONLY:
            self.get_connections()

    def get_connections(self):
        oci.regions.enable_instance_metadata_service()
        self.CLIENT = oci.identity.IdentityClient(self.CONFIG_FROM_FILE)
        self.TENANTNAME = self.CLIENT.get_tenancy(tenancy_id=self.OCID).data.name
        return self.CLIENT, self.TENANTNAME

    def get_region_cache(self, profile_name='DEFAULT'):
        REGION_CACHE = {}
        if profile_name not in self.CONFIGSTORE.PROFILES:
            self.CONFIGSTORE.create_profile(profile_name)
        TIMESTAMP = str(datetime.datetime.now().timestamp())
        self.get_connections()
        Log.info("caching region subscriptions...")
        REGION_SUBSCRIPTIONS = self.CLIENT.list_region_subscriptions(tenancy_id=self.OCID).data
        for DATA in REGION_SUBSCRIPTIONS:
            IS_HOME = DATA.is_home_region
            KEY = DATA.region_key
            NAME = DATA.region_name
            STATUS = DATA.status
            REGION_CACHE[NAME] = {
                 'region_key': KEY,
                 'region_name': NAME,
                 'status': STATUS,
                 'is_home_region': IS_HOME,
            }
        DATADICT = {}
        DATA = []
        for I in REGION_CACHE:
            if 'last_cache_update' not in I:
                DATA.append(REGION_CACHE[I])
            if DATA:
                 DATADICT = DATA
                 self.CONFIGSTORE.update_metadata(REGION_CACHE, 'cached_regions', profile_name, True)
                 self.CONFIGSTORE = Config('ocitools')

    def get_cached_regions(self, profile_name):
        if profile_name in self.CONFIGSTORE.PROFILES:
            if 'cached_regions' in self.CONFIGSTORE.PROFILES[profile_name]['metadata']:
                return self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_regions']
        else:
            return None

    def refresh(self, type, profile_name):
        if type == 'cached_regions':
            self.get_region_cache(profile_name)
        self.CONFIGSTORE = Config('ocitools')

    def describe(self, profile_name):
        self.get_connections()
        # Get the current user
        USER = self.CLIENT.get_user(self.CONFIG_FROM_FILE["user"]).data
        TENANCY = self.CLIENT.get_tenancy(tenancy_id=self.OCID).data
        return USER, TENANCY
   
    def list_regions(self, profile_name):
        if not self.CLIENT:
            self.get_connections()
        return self.CLIENT.list_regions().data 



from toolbox.logger import Log
from toolbox.misc import detect_environment
from configstore.configstore import Config
from .oci_config import OCIconfig
from . import iam_nongc
import datetime
from tabulate import tabulate
import os, oci, json
from . import shapes

CONFIGSTORE = Config('ocitools')

class DBSclient():

    def __init__(self, profile_name, region='us-ashburn-1', cache_only=False):
        self.CONFIGSTORE = Config('ocitools')
        self.CONFIG = OCIconfig()
        self.OCI_REGION = region

        # get these from oci.config based on profile name and validate the config
        self.config = oci.config.from_file("~/.oci/config", profile_name)
        self.OCID = self.config.get("tenancy")
        self.user = self.config.get("user")
        self.fingerprint = self.config.get("fingerprint")
        self.keyfile = self.config.get("key_file")
        self.CONFIG_FROM_FILE = {
                'tenancy': self.OCID,
                'user': self.user,
                'fingerprint': self.fingerprint,
                'key_file': self.keyfile,
                'region': self.OCI_REGION
        }
        oci.config.validate_config(self.CONFIG_FROM_FILE)
        self.CACHE_UPDATE_INTERVAL = 60*60*24*7 # update cache every week
        self.OCI_PROFILE = profile_name
        self.CACHE_ONLY = cache_only
        if not self.CACHE_ONLY:
            self.get_connections()

    def get_connections(self):
        self.CLIENT = oci.identity.IdentityClient(self.CONFIG_FROM_FILE)
        self.TENANTNAME = self.CLIENT.get_tenancy(tenancy_id=self.OCID).data.name
        self.COMPUTE = oci.core.ComputeClient(self.CONFIG_FROM_FILE)
        self.NETWORK = oci.core.VirtualNetworkClient(self.CONFIG_FROM_FILE)
        self.DBS = oci.database.DatabaseClient(self.CONFIG_FROM_FILE)
        return self.CLIENT, self.COMPUTE, self.NETWORK, self.TENANTNAME

    def show_dbs_instances(self, profile_name):
        CACHE = self.get_cached_dbs_instances(profile_name)
        if CACHE is not None:
            return CACHE

    def DisplayInstances(self, instances, compartmentName, instancetype, region_name):

        TIMESTAMP = str(datetime.datetime.now().timestamp())
        AllPredefinedTags = True 
        NoValueString = "n/a"
        FieldSeperator = ","
        EndLine = "\n"
        customertags = []
        DBS_CACHE = {}
        ATP_CACHE = {}
        ADW_CACHE = {}

        for instance in instances:
            privateips = ""
            publicips = ""
            instancetypename = ""
            tagtxt = ""
            OS = ""
            # Handle details for Database Instances
            if instancetype == "DB":
                OCPU, MEM, SSD = shapes.ComputeShape(instance.shape)
                OCPU = instance.cpu_core_count # Overwrite Shape's CPU count, with DB enabled CPU count
                response = self.DBS.list_db_nodes(compartment_id=instance.compartment_id,db_system_id=instance.id)
                dbnodes = response.data
                try:
                    for dbnode in dbnodes:
                        responsenic = NetworkClient.get_vnic(vnic_id=dbnode.vnic_id)
                        nicinfo = responsenic.data
                        privateips = privateips + nicinfo.private_ip + " "
                        publicips = publicips + nicinfo.public_ip + " "
                except:
                    privateips = NoValueString
                    publicips = NoValueString

                instancetypename = "DB " + instance.database_edition
                OS = "Oracle Linux 6.8"
                shape = instance.shape
                prefix, AD = instance.availability_domain.split(":")

                DBS_CACHE[instance.display_name] = {
                                'display_name': instance.display_name,
                                'lifecycle_state': instance.lifecycle_state,
                                'ocpu': OCPU,
                                'shape': shape,
                                'compartment_name': compartmentName,
                                'type': instancetypename,
                                'public_ips': publicips,
                                'private_ips': privateips,
                                'OS': OS,
                                'AD': AD
                }
                DBS_CACHE['last_cache_update'] = TIMESTAMP

            # Handle details for Autonomous Database (ATP)
            if instancetype == "ATP":
                OCPU = instance.cpu_core_count
                MEM = NoValueString
                SSD = instance.data_storage_size_in_tbs
                instancetypename = "ATP"
                OS = NoValueString
                shape = "ATP"
                AD = region_name.upper()
                privateips = NoValueString
                publicips = NoValueString

                ATP_CACHE[instance.db_name] = {
                                'display_name': instance.db_name,
                                'lifecycle_state': instance.lifecycle_state,
                                'ocpu': OCPU,
                                'memory': MEM,
                                'shape': shape,
                                'compartment_name': compartmentName,
                                'type': instancetypename,
                                'public_ips': publicips,
                                'private_ips': privateips,
                                'OS': OS,
                                'AD': AD
                }
                ATP_CACHE['last_cache_update'] = TIMESTAMP

            # Handle details for Autonomous Database (ADW)
            if instancetype == "ADW":
                OCPU = instance.cpu_core_count
                MEM = NoValueString
                SSD = instance.data_storage_size_in_tbs
                instancetypename = "ADW"
                OS = NoValueString
                shape = "ADW"
                AD = region_name.upper()
                privateips = NoValueString
                publicips = NoValueString

                ADW_CACHE[instance.display_name] = {
                                'display_name': instance.display_name,
                                'lifecycle_state': instance.lifecycle_state,
                                'ocpu': OCPU,
                                'memory': MEM,
                                'shape': shape,
                                'compartment_name': compartmentName,
                                'type': instancetypename,
                                'public_ips': publicips,
                                'private_ips': privateips,
                                'OS': OS,
                                'AD': AD
                }
                ADW_CACHE['last_cache_update'] = TIMESTAMP
            try:
                namespaces = instance.defined_tags
                for customertag in customertags:
                    try:
                        tagtxt = tagtxt + FieldSeperator + namespaces[customertag[0]][customertag[1]]
                    except:
                        tagtxt = tagtxt + FieldSeperator + NoValueString
            except:
                tagtxt = ""  # No Tags
        return DBS_CACHE, ADW_CACHE, ATP_CACHE

    def get_region_cache(self, profile_name='DEFAULT'):
        REGION_CACHE = {}
        if profile_name not in self.CONFIGSTORE.PROFILES:
            self.CONFIGSTORE.create_profile(profile_name)
        TIMESTAMP = str(datetime.datetime.now().timestamp())
        self.get_connections()
        Log.info("caching region subscriptions...")
        REGION_SUBSCRIPTIONS = self.CLIENT.list_region_subscriptions(tenancy_id=self.OCID).data
        for DATA in REGION_SUBSCRIPTIONS:
            IS_HOME = DATA.is_home_region
            KEY = DATA.region_key
            NAME = DATA.region_name
            STATUS = DATA.status
            REGION_CACHE[KEY] = {
                 'region_key': KEY,
                 'region_name': NAME,
                 'status': STATUS,
                 'is_home_region': IS_HOME,
            }
        DATADICT = {}
        DATA = []
        for I in REGION_CACHE:
            if 'last_cache_update' not in I:
                DATA.append(REGION_CACHE[I])
            if DATA:
                 DATADICT = DATA
                 self.CONFIGSTORE.update_metadata(REGION_CACHE, 'cached_regions', profile_name, True)
                 self.CONFIGSTORE = Config('ocitools')

    def get_dbs_instances(self, profile_name='DEFAULT'):

        if profile_name not in self.CONFIGSTORE.PROFILES:
            self.CONFIGSTORE.create_profile(profile_name)
        TIMESTAMP = str(datetime.datetime.now().timestamp())
        self.get_connections()
        Log.info("caching DBS instances across all compartments...")
        response = oci.pagination.list_call_get_all_results(self.CLIENT.list_compartments,self.OCID,compartment_id_in_subtree=True)
        compartments = response.data
        RootCompartment = oci.identity.models.Compartment()
        RootCompartment.id = self.OCID
        RootCompartment.name = "root"
        RootCompartment.lifecycle_state = "ACTIVE"
        compartments.insert(0, RootCompartment)
        for compartment in compartments:
            compartmentName = compartment.name
            if compartment.lifecycle_state == "ACTIVE":
                Log.debug("processing compartment:  " + compartmentName)
                compartmentID = compartment.id
                try:
                    response = oci.pagination.list_call_get_all_results(self.DBS.list_instances,compartment_id=compartmentID)
                    if len(response.data) > 0:
                        DBS_CACHE, ADW_CACHE, ATP_CACHE = self.DisplayInstances(response.data, compartmentName, "DB", self.OCI_REGION)
                        DATADICT = {}
                        DATA = []
                        for I in DBS_CACHE:
                            if 'last_cache_update' not in I:
                                DATA.append(DBS_CACHE[I])
                        if DATA:
                            DATADICT = DATA
                            DICT = {}
                            DICT[self.OCI_REGION] = DBS_CACHE
                            self.CONFIGSTORE.update_metadata(DICT, 'cached_dbs_instances', profile_name, True)
                            self.CONFIGSTORE = Config('ocitools')
                except:
                    pass

                try:
                    response = oci.pagination.list_call_get_all_results(self.DBS.list_autonomous_data_warehouses,compartment_id=compartmentID)
                    if len(response.data) > 0:
                        DBS_CACHE, ADW_CACHE, ATP_CACHE = self.DisplayInstances(response.data, compartmentName, "ADW", self.OCI_REGION)
                        DATADICT = {}
                        DATA = []
                        for I in ADW_CACHE:
                            if 'last_cache_update' not in I:
                                DATA.append(ADW_CACHE[I])
                        if DATA:
                            DATADICT = DATA
                            DICT = {}
                            DICT[self.OCI_REGION] = ADW_CACHE
                            self.CONFIGSTORE.update_metadata(DICT, 'cached_dbs_instances', profile_name, True)
                            self.CONFIGSTORE = Config('ocitools')
                except:
                    pass

                try:
                    response = oci.pagination.list_call_get_all_results(self.DBS.list_autonomous_databases,compartment_id=compartmentID)
                    if len(response.data) > 0:
                        DBS_CACHE, ADW_CACHE, ATP_CACHE = self.DisplayInstances(response.data, compartmentName, "ATP", self.OCI_REGION)
                        DATADICT = {}
                        DATA = []
                        for I in ATP_CACHE:
                            if 'last_cache_update' not in I:
                                DATA.append(ATP_CACHE[I])
                        if DATA:
                            DATADICT = DATA
                            DICT = {}
                            DICT[self.OCI_REGION] = ATP_CACHE
                            self.CONFIGSTORE.update_metadata(DICT, 'cached_dbs_instances', profile_name, True)
                            self.CONFIGSTORE = Config('ocitools')
                except:
                    pass

    def get_cached_dbs_instances(self, profile_name):
        if profile_name in self.CONFIGSTORE.PROFILES:
            if 'cached_dbs_instances' in self.CONFIGSTORE.PROFILES[profile_name]['metadata']:
                return self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_dbs_instances']
        else:
            return None

    def refresh(self, type, profile_name):
        if type == 'cached_dbs_instances':
            self.get_dbs_instances(profile_name)
        self.CONFIGSTORE = Config('ocitools')

    def auto_refresh(self, profile_name):
        TIME_NOW = datetime.datetime.now().timestamp()
        try:    
            DBS_INSTANCES_TS = self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_dbs_instances']['last_cache_update']
        except KeyError:
            DBS_INSTANCES_TS = 0    
        if TIME_NOW - float(DBS_INSTANCES_TS) > self.CACHE_UPDATE_INTERVAL:
            Log.info('automatic refresh of dbs instance cache initiated')
            self.refresh('cached_dbs_instances', profile_name)

    def get_region_from_profile(self, profile_name):
        REGION = self.CONFIG.get_from_config('config', 'region', profile_name=profile_name)
        if REGION is None:
            Log.critical("Please run goat oci iam authenticate for the target profile before using the oss module")
        return REGION



from toolbox.logger import Log
from toolbox.misc import detect_environment
from configstore.configstore import Config
from .oci_config import OCIconfig
from . import iam_nongc
import datetime
from tabulate import tabulate
import os, oci, json

CONFIGSTORE = Config('ocitools')

class IAMclient():

    def __init__(self, profile_name, region='us-ashburn-1', cache_only=False):
        self.CONFIGSTORE = Config('ocitools')
        self.CONFIG = OCIconfig()
        self.CACHE_UPDATE_INTERVAL = 60*60*24*7 # update cache every week
        self.OCI_PROFILE = profile_name
        self.OCI_REGION = region
        self.CACHE_ONLY = cache_only

        # get these from oci.config based on profile name and validate the config
        self.config = oci.config.from_file("~/.oci/config", profile_name)
        self.OCID = self.tenant = self.config.get("tenancy")
        self.user = self.config.get("user")
        self.fingerprint = self.config.get("fingerprint")
        self.keyfile = self.config.get("key_file")
        self.CONFIG_FROM_FILE = {
                'tenancy': self.tenant,
                'user': self.user,
                'fingerprint': self.fingerprint,
                'key_file': self.keyfile,
                'region': self.OCI_REGION
        }
        try:
            oci.config.validate_config(self.CONFIG_FROM_FILE)
        except:
            Log.critical('unable to setup a proper oci configuration file')

        if not self.CACHE_ONLY:
            self.get_connections()

    def get_connections(self):
        self.CLIENT = oci.identity.IdentityClient(self.CONFIG_FROM_FILE)
        self.TENANTNAME = self.CLIENT.get_tenancy(tenancy_id=self.OCID).data.name
        self.VAULT = oci.vault.VaultsClient(self.CONFIG_FROM_FILE)
        self.SECRETS = oci.secrets.SecretsClient(self.CONFIG_FROM_FILE)
        self.KMS_VAULT = oci.key_management.KmsVaultClient(self.CONFIG_FROM_FILE)
        self.COMPOSITE = oci.key_management.KmsVaultClientCompositeOperations(self.KMS_VAULT)
        return self.CLIENT, self.TENANTNAME, self.VAULT, self.SECRETS, self.KMS_VAULT, self.COMPOSITE

    def get_compartments(self):
        try:
            self.get_connections()
        except:
            return None
        COMPARTMENTS = self.CLIENT.list_compartments(self.OCID, compartment_id_in_subtree=True).data
        ROOT_COMPARTMENT = self.CLIENT.get_compartment(compartment_id=self.OCID).data
        COMPARTMENTS.append(ROOT_COMPARTMENT)
        return COMPARTMENTS

    def get_region_from_profile(self, profile_name):
        REGION = self.CONFIG.get_from_config('config', 'region', profile_name=profile_name)
        if REGION is None:
            Log.critical("Please run goat oci iam authenticate for the target profile before using the oss module")
        return REGION

    def get_cached_compartments(self, profile_name):
        if profile_name in self.CONFIGSTORE.PROFILES:
            if 'cached_compartments' in self.CONFIGSTORE.PROFILES[profile_name]['metadata']:
                return self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_compartments'][self.OCI_REGION]
        else:
            return None

    def refresh(self, type, profile_name):
        if type == 'cached_compartments':
            self.get_compartments_cache(profile_name)
        self.CONFIGSTORE = Config('ocitools')

    def get_compartments_cache(self, profile_name='DEFAULT'):
        CACHE = {}
        if profile_name not in self.CONFIGSTORE.PROFILES:
            self.CONFIGSTORE.create_profile(profile_name)
        TIMESTAMP = str(datetime.datetime.now().timestamp())
        self.get_connections()
        Log.info("caching compartment data across all levels...")
        COMPARTMENTS = self.get_compartments()
        for COMPARTMENT in COMPARTMENTS:
            DESC = COMPARTMENT.description
            ID = COMPARTMENT.id
            STATE = COMPARTMENT.lifecycle_state
            NAME = COMPARTMENT.name
            CACHE[ID] = {
                'name': NAME,
                'id': ID,
                'lifecycle_state': STATE,
                'description': DESC
            }
        CACHE['last_cache_update'] = TIMESTAMP
        DICT = {}
        DICT[self.OCI_REGION] = CACHE
        self.CONFIGSTORE.update_metadata(DICT, 'cached_compartments', profile_name, True)

    def describe(self, comp_ocid, profile_name):
        try:
            self.get_connections()
        except:
            return None
        RESPONSE = self.CLIENT.get_compartment(compartment_id=comp_ocid).data
        DICT = {}
        DICT[comp_ocid] = RESPONSE
        return DICT

    def create_compartment(self, comp_ocid, name, description):
        RESPONSE = self.CLIENT.create_compartment(create_compartment_details=oci.identity.models.CreateCompartmentDetails(compartment_id=comp_ocid, name=name, description=description)).data
        return RESPONSE

    def delete_compartment(self, comp_ocid):
        RESPONSE = self.CLIENT.delete_compartment(compartment_id=comp_ocid).headers
        return RESPONSE




import click, os, json
from toolbox.logger import Log
from .iamclient import IAMclient
from tabulate import tabulate
from configstore.configstore import Config
from toolbox.misc import set_terminal_width
from .iam import get_latest_profile
from toolbox.menumaker import Menu
from datetime import datetime
from datetime import timedelta

CONFIG = Config('ocitools')
IGNORE = ['latest']

@click.group('compartment', invoke_without_command=True, help='module to manage compartments for tenant', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.option('-m', '--menu', help='use the menu to perform compartment actions', is_flag=True, show_default=True, default=False, required=False)
@click.pass_context
def compartment(ctx, menu):
    profile_name = ctx.obj['PROFILE']
    if menu:
        ctx.obj['MENU'] = True
    else:
        ctx.obj['MENU'] = False
    pass

@compartment.command(help='manually refresh comparments stored in cache', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
def refresh(ctx):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _refresh('cached_compartments', oci_region, profile_name)

def _refresh(cache_type, oci_region, profile_name):
    try:
        if cache_type == 'cached_compartments':
            IAM = get_IAMclient(profile_name, oci_region, auto_refresh=False, cache_only=False)
            RESULT = IAM.refresh('cached_compartments', profile_name)
        return True
    except:
        False

@compartment.command(help='create OCI compartment for region and tenant', context_settings={'help_option_names':['-h','--help']})
@click.argument('compartment', required=True)
@click.option('-d', '--description', help='provide a short description for the new compartment', required=False, type=str, default=None)
@click.pass_context
def create(ctx, compartment, description):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _create(ctx, profile_name, oci_region, compartment, description)

def _create(ctx, profile_name, oci_region, compartment,  description):
    IAM = get_IAMclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
    for PROFILE in CONFIG.PROFILES:
        if PROFILE in IGNORE:
            continue
        CACHED_COMPARTMENTS = IAM.get_compartments()
        DATA = []
        for COMPARTMENT in CACHED_COMPARTMENTS:
            if COMPARTMENT.lifecycle_state != 'ACTIVE':
                continue
            COMP_NAME = str(COMPARTMENT.name).ljust(50)
            COMP_OCID = str(COMPARTMENT.id).ljust(100)
            STR = COMP_OCID + '\t' + COMP_NAME
            DATA.append(STR)
    INPUT = f'Compartments => {profile_name}'
    CHOICE = runMenu(DATA, INPUT)
    if CHOICE:
        CHOICE = ''.join(CHOICE)
        OCID = CHOICE.split('\t')[0].strip()
        NAME = CHOICE.split('\t')[1].strip()
    else:
        Log.critical('please choose a compartment')

    Log.info(f'creating compartment {compartment} in compartment {NAME}')
    if description is None: description = compartment
    RESPONSE = IAM.create_compartment(OCID, compartment, description)
    if RESPONSE:
        Log.info(f'{compartment} has been created in compartment level {NAME}')
    else:
        Log.critical(f'problem while creating compartment {compartment}')
    
@compartment.command(help='menu-guided solution to delete OCI compartment\'s for region and tenant', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
def delete(ctx):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _delete(ctx, profile_name, oci_region)

def _delete(ctx, profile_name, oci_region):
    IAM = get_IAMclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
    for PROFILE in CONFIG.PROFILES:
        if PROFILE in IGNORE:
            continue
        CACHED_COMPARTMENTS = IAM.get_compartments()
        DATA = []
        for COMPARTMENT in CACHED_COMPARTMENTS:
            if COMPARTMENT.lifecycle_state != 'ACTIVE':
                continue
            COMP_NAME = str(COMPARTMENT.name).ljust(50)
            COMP_OCID = str(COMPARTMENT.id).ljust(100)
            STR = COMP_OCID + '\t' + COMP_NAME
            DATA.append(STR)
    INPUT = f'Compartments => {profile_name}'
    CHOICE = runMenu(DATA, INPUT)
    if CHOICE:
        CHOICE = ''.join(CHOICE)
        OCID = CHOICE.split('\t')[0].strip()
        NAME = CHOICE.split('\t')[1].strip()
    else:
        Log.critical('please choose a compartment for deletion')
    Log.info(f'deleting compartment {NAME}')
    RESPONSE = IAM.delete_compartment(OCID)
    if RESPONSE:
        Log.info(f'{NAME} has been deleted with response:')
        print(RESPONSE)
    else:
        Log.critical(f'problem while deleting compartment {NAME}; response is: {RESPONSE}')

@compartment.command(help='show the data stored in cached compartments', context_settings={'help_option_names':['-h','--help']})
@click.argument('compartment', required=False)
@click.pass_context
def show(ctx, compartment):
    profile_name = ctx.obj['PROFILE']
    _show(ctx, profile_name, compartment)

def _show(ctx, profile_name, compartment):
    if not compartment:
        if ctx.obj["MENU"]:
            DATA = []
            DICT = {}
            IAM = get_IAMclient(profile_name, auto_refresh=False, cache_only=True)
            RESPONSE = IAM.get_cached_compartments(profile_name)
            for I in RESPONSE:
                if I not in 'last_cache_update':
                    NAME = RESPONSE[I]['name'].ljust(50)
                    DATA.append(I.ljust(100) + '\t' + NAME)
            INPUT = 'Compartment Manager'
            CHOICE = runMenu(DATA, INPUT)
            try:
                CHOICE = ''.join(CHOICE)
                OCID = CHOICE.split('\t')[0].strip()
            except:
                Log.critical("please select a compartment to continue...")
            Log.info(f"describing {OCID}:")
            RESPONSE = IAM.describe(OCID, profile_name)
            for I in RESPONSE:
                print(RESPONSE[I])
        else:
            IAM = get_IAMclient(profile_name, auto_refresh=False, cache_only=True)
            RESPONSE = IAM.get_cached_compartments(profile_name)
            show_as_table(RESPONSE)
    else:
        IAM = get_IAMclient(profile_name, auto_refresh=False, cache_only=True)
        RESPONSE = IAM.describe(compartment)
        Log.info(f"describing {compartment}:\n" + json.dumps(RESPONSE, indent=2, sort_keys=True, default=str))

def show_as_table(source_data):
    IGNORE = ['last_cache_update']
    DATADICT = {}
    DATA = []
    try:
        for I in source_data:
            if I not in IGNORE:
                DATA.append(source_data[I])
                if DATA:
                    DATADICT = DATA
        if DATADICT != []:
            Log.info(f"\n{tabulate(DATADICT, headers='keys', tablefmt='rst')}\n")
    except:
        return None

def get_IAMclient(profile_name, region='us-ashburn-1', auto_refresh=False, cache_only=False):
    CLIENT = IAMclient(profile_name, region, cache_only)
    return CLIENT

def runMenu(DATA, INPUT):
    COUNT = 0
    FINAL = []
    TITLE = f'Compartment Menu: {INPUT}'
    for data in DATA:
        COUNT = COUNT + 1
        RESULTS = []
        RESULTS.append(data)
        FINAL.append(RESULTS)
    SUBTITLE = f'showing {COUNT} available object(s)'
    JOINER = '\t\t'
    FINAL_MENU = Menu(FINAL, TITLE, JOINER, SUBTITLE)
    CHOICE = FINAL_MENU.display()
    return CHOICE

def get_region(ctx, profile_name):
    OCI_REGION = ctx.obj['REGION']
    if not OCI_REGION:
        IAM = get_IAMclient(profile_name)
        OCI_REGION = IAM.get_region_from_profile(profile_name)
    return OCI_REGION



import os, click
from toolbox.logger import Log
from . import iam_nongc
import toolbox.misc as misc
from configstore.configstore import Config
from .oci_config import OCIconfig

@click.group('iam', help='manage and switch between OCI profiles for all realms', context_settings={'help_option_names':['-h','--help'], 'max_content_width': misc.set_terminal_width()})
def iam():
    pass

def get_latest_profile():

    CONFIGSTORE = Config('ocitools')
    LATEST = CONFIGSTORE.get_profile('latest')
    if LATEST is None:
        PROFILE = 'default'
    else:
        PROFILE = LATEST['config']['name']
    return PROFILE

def get_latest_region(profile_name):
    CONFIG = OCIconfig()
    REGION = CONFIG.get_from_config('config', 'region', profile_name=profile_name)
    if REGION is None:
        return 'us-ashburn-1'
    return REGION

iam.add_command(iam_nongc.authenticate)



import click, json, sys, os, oci
from toolbox.logger import Log
from .ossclient import OSSclient, setup_config_file
from configstore.configstore import Config
from toolbox.menumaker import Menu
from toolbox.misc import set_terminal_width
from .iam import get_latest_profile
from pprint import pprint

CONFIG = Config('ocitools')
ignore = ['latest']

@click.group('oss', invoke_without_command=True, help='object storage functions to sync buckets and filesystems', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.option('-m', '--menu', help='use the menu to perform OCI OSS actions', is_flag=True, show_default=True, default=False, required=False)
@click.pass_context
def oss(ctx, menu):
    if menu:
        ctx.obj['MENU'] = True
    else:
        ctx.obj['MENU'] = False

@oss.command(help='create a new bucket', context_settings={'help_option_names':['-h','--help']})
@click.argument('bucket_name', required=True)
@click.pass_context
def create(ctx, bucket_name):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    OSS = get_OSSclient(profile_name, oci_region)
    for PROFILE in CONFIG.PROFILES:
        if PROFILE in ignore:
            continue
        CACHED_COMPARTMENTS = OSS.get_compartments()
        DATA = []
        for COMPARTMENT in CACHED_COMPARTMENTS:
            COMP_NAME = str(COMPARTMENT.name).ljust(50)
            COMP_OCID = str(COMPARTMENT.id).ljust(100)
            STR = COMP_OCID + '\t' + COMP_NAME
            DATA.append(STR)
    INPUT = f'Compartments => {profile_name}'
    CHOICE = runMenu(DATA, INPUT)
    if CHOICE:
        CHOICE = ''.join(CHOICE)
        ocid = CHOICE.split('\t')[0].strip()
    else:
        Log.critical('please choose a compartment')
    _create(profile_name, bucket_name, ocid, oci_region)
    Log.info('refreshing cache now for buckets after creation')
    _refresh(profile_name, oci_region)

def _create(profile_name, bucket_name, ocid, oci_region):
    MYOSS = get_OSSclient(profile_name, oci_region)
    config = setup_config_file(profile_name, oci_region)
    object_storage = oci.object_storage.ObjectStorageClient(config)
    namespace = object_storage.get_namespace(compartment_id=ocid).data
    RESULT = MYOSS.create_bucket(ocid, namespace, bucket_name, profile_name)
    if RESULT:
        Log.info(f"bucket: {bucket_name} created in region: {oci_region} successfully.")
        return RESULT
    else:
        Log.critical("failed to create the bucket")

@oss.command(help='download from OSS to local storage', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
@click.argument('source', required=False, default=None)
@click.option('--destination', help="provide a local destination for download", type=str, required=False, default=os.environ["HOME"], show_default=True)
def download(ctx, source, destination):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    if ctx.obj["MENU"] or source is None:
        if source is None:
            CACHED_BUCKETS = {}
            CACHED_CONTENTS = []
            for PROFILE in CONFIG.PROFILES:
                if PROFILE in ignore:
                    continue
                if PROFILE == profile_name:
                    try:
                        CACHED_BUCKETS.update(CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region))
                        CACHED_BUCKETS.pop('last_cache_update', None)
                    except:
                        Log.critical(f'there are 0 buckets cached for {oci_region} and profile {PROFILE}')
            INPUT = f'Downloads => {profile_name}'
            CHOICE = runMenu(CACHED_BUCKETS, INPUT)
            if CHOICE:
                source = ''.join(CHOICE)
                for PROFILE in CONFIG.PROFILES:
                    if PROFILE in ignore:
                        continue
                    if CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region) and source in CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region):
                        profile_name = PROFILE
                        OSS = get_OSSclient(PROFILE, oci_region)
                        CONTENTS = OSS.show_bucket_content(source, profile_name)
                        if type(CONTENTS) is not bool:
                            CACHED_CONTENTS = CONTENTS
                            INPUT = f'Bucket Contents => {profile_name}'
                            CHOICE = runMenu(CACHED_CONTENTS, INPUT)
                            if CHOICE:
                                CHOICE = ''.join(CHOICE)
                                if _download(profile_name, oci_region, source, destination, CHOICE):
                                    Log.info(f"downloading {CHOICE} from bucket {source} completed")
                                else:
                                    Log.critical(f"failed to download {CHOICE} from OSS bucket {source} to {destination}")
                            else:
                                Log.critical(f"please select a valid choice to download, exiting.")
                        break
        else:
            CACHED_CONTENTS = []
            for PROFILE in CONFIG.PROFILES:
                if PROFILE in ignore:
                    continue
                if CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region) and source in CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region):
                    profile_name = PROFILE
                    OSS = get_OSSclient(PROFILE, oci_region)
                    CONTENTS = OSS.show_bucket_content(source, profile_name)
                    if type(CONTENTS) is not bool:
                        CACHED_CONTENTS = CONTENTS
                        INPUT = f'Bucket Contents => {profile_name}'
                        CHOICE = runMenu(CACHED_CONTENTS, INPUT)
                        if CHOICE:
                            CHOICE = ''.join(CHOICE)
                            if _download(profile_name, oci_region, source, destination, CHOICE):
                                Log.info(f"downloading {CHOICE} from bucket {source} completed")
                            else:
                                Log.critical(f"failed to download {CHOICE} from OSS bucket {source} to {destination}")
                        else:
                            Log.critical(f"please select a valid choice to download, exiting.")
                    break
    else:
        for profile_name in CONFIG.PROFILES:
            if profile_name in ignore:
                continue
            PROFILE = profile_name
            if CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region) and source in CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region):
                OSS = get_OSSclient(PROFILE, oci_region)
                CONTENTS = OSS.show_bucket_content(source, profile_name)
                if type(CONTENTS) is not bool:
                    CACHED_CONTENTS = CONTENTS
                    INPUT = f'Bucket Contents => {profile_name}'
                    CHOICE = runMenu(CACHED_CONTENTS, INPUT)
                    if CHOICE:
                        CHOICE = ''.join(CHOICE)
                        if _download(profile_name, oci_region, source, destination, CHOICE):
                            Log.info(f"downloading {CHOICE} from bucket {source} completed")
                        else:
                            Log.critical(f"failed to download {CHOICE} from OSS bucket {source} to {destination}")
                    else:
                        Log.critical(f"please select a valid choice to download, exiting.")
                break

def _download(profile_name, oci_region, source, destination, filename):
    OSS = get_OSSclient(profile_name, oci_region)
    BUCKETS = CONFIG.PROFILES[profile_name]['metadata']['cached_buckets'][oci_region]
    for BUCKETNAME in BUCKETS:
        if BUCKETNAME == source:
            NAMESPACE = BUCKETS[BUCKETNAME]['namespace']
    try:
        OSS.oss_to_local(source, destination, filename, NAMESPACE)
        return True
    except:
        return False

@oss.command(help='upload from local storage to OSS bucket', context_settings={'help_option_names':['-h','--help']})
@click.option('--source', help="provide a local source for upload", type=str, required=True)
@click.argument('destination', required=False, default=None)
@click.pass_context
def upload(ctx, source, destination):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    CACHED_BUCKETS = {}
    if destination is None:
        for PROFILE in CONFIG.PROFILES:
            if PROFILE == profile_name:
                try:
                    CACHED_BUCKETS.update(CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region))
                    CACHED_BUCKETS.pop('last_cache_update', None)
                except:
                    Log.critical(f'there are 0 buckets cached for {oci_region} and profile {PROFILE}')
        INPUT = f'Uploads => {profile_name}'
        CHOICE = runMenu(CACHED_BUCKETS, INPUT)
        if CHOICE:
            destination = ''.join(CHOICE)
            for profile_name in CONFIG.PROFILES:
                if profile_name in ignore:
                    continue
                if destination in CONFIG.get_metadata_aws('cached_buckets', profile_name, oci_region):
                    if _upload(profile_name, oci_region, source, destination):
                        Log.info(f"uploading local files from {source} to OSS bucket {destination} completed")
                    else:
                        Log.critical(f"failed to upload local files from {source} to OSS bucket {destination}")
                    break
    else:
        for PROFILE in CONFIG.PROFILES:
            if PROFILE in ignore:
                continue
            if destination in CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region):
                profile_name = PROFILE
                if _upload(profile_name, oci_region, source, destination):
                    Log.info(f"uploading local files from {source} to OSS bucket {destination} completed")
                else:
                    Log.critical(f"failed to upload local files from {source} to OSS bucket {destination}")
                break

def _upload(profile_name, oci_region, source, destination):
    OSS = get_OSSclient(profile_name, oci_region)
    BUCKETS = CONFIG.PROFILES[profile_name]['metadata']['cached_buckets'][oci_region]
    for BUCKETNAME in BUCKETS:
        if BUCKETNAME == destination:
            NAMESPACE = BUCKETS[BUCKETNAME]['namespace']
    try:
        OSS.local_to_oss(destination, source, NAMESPACE)
        return True
    except:
        return False

@oss.command(help='delete a specified bucket', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
@click.argument('bucket', required=False, default=None)
def delete(ctx, bucket):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    if bucket is None:
        if bucket is None:
            CACHED_BUCKETS = {}
            for PROFILE in CONFIG.PROFILES:
                if PROFILE in ignore:
                    continue
                if PROFILE == profile_name:
                    try:
                        CACHED_BUCKETS.update(CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region))
                        CACHED_BUCKETS.pop('last_cache_update', None)
                    except:
                        Log.warn(f'there are 0 buckets in {oci_region} and profile {PROFILE}')
                        sys.exit()
            INPUT = f'Deletion => {profile_name}'
            CHOICE = runMenu(CACHED_BUCKETS, INPUT)
            if CHOICE:
                bucket = ''.join(CHOICE)
                for PROFILE in CONFIG.PROFILES:
                    if PROFILE in ignore:
                        continue
                    if bucket in CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region):
                        profile_name = PROFILE
                        _delete(bucket, profile_name, oci_region)
                        Log.info('refreshing cache now for buckets after deletion')
                        _refresh(profile_name, oci_region)
                        break
        else:
            for PROFILE in CONFIG.PROFILES:
                if PROFILE in ignore:
                    continue
                if bucket in CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region):
                    profile_name = PROFILE
                    _delete(bucket, profile_name, oci_region)
                    Log.info('refreshing cache now for buckets after deletion')
                    _refresh(profile_name, oci_region)
                    break
    else:
        for PROFILE in CONFIG.PROFILES:
            if PROFILE in ignore:
                continue
            if bucket in CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region):
                profile_name = PROFILE
                _delete(ctx, bucket, profile_name, oci_region)
                Log.info('refreshing cache now for buckets after deletion')
                _refresh(profile_name, oci_region)
                break

def _delete(bucket, profile_name, oci_region):
    OSS = get_OSSclient(profile_name, oci_region)
    BUCKETS = CONFIG.PROFILES[profile_name]['metadata']['cached_buckets'][oci_region]
    for BUCKETNAME in BUCKETS:
        if BUCKETNAME == bucket:
            NAMESPACE = BUCKETS[BUCKETNAME]['namespace']
    RESULT = OSS.delete_bucket(NAMESPACE, bucket, profile_name)
    if RESULT:
        Log.info(f"bucket: {bucket} deleted in region: {oci_region} successfully.")
        return True
    else:
        Log.critical("failed to delete the bucket")

@oss.command(help='manually refresh OSS cached data', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
def refresh(ctx):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _refresh(profile_name, oci_region)

def _refresh(profile_name, oci_region):
    try:
        OSS = get_OSSclient(profile_name, oci_region, auto_refresh=False)
        OSS.cache_buckets(profile_name)
        return True
    except:
        return False

@oss.command(help='show the data stored in OSS cache', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
@click.argument('bucket', required=False, default=None)
def show(ctx, bucket):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    if ctx.obj["MENU"]:
        if bucket is None:
            CACHED_BUCKETS = {}
            for PROFILE in CONFIG.PROFILES:
                if PROFILE in ignore:
                   continue
                if PROFILE == profile_name:
                    try:
                        CACHED_BUCKETS.update(CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region))
                        CACHED_BUCKETS.pop('last_cache_update', None)
                    except:
                        Log.critical(f"there are 0 cached buckets for {PROFILE}")
            INPUT = f'Bucket List => {profile_name}'
            CHOICE = runMenu(CACHED_BUCKETS, INPUT)
            if CHOICE:
                CACHED_CONTENTS = []
                for PROFILE in CONFIG.PROFILES:
                    if PROFILE in ignore:
                        continue
                    if CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region) and ''.join(CHOICE) in CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region):
                        OSS = get_OSSclient(PROFILE, oci_region)
                        CONTENTS = OSS.show_bucket_content(''.join(CHOICE), profile_name)
                        if type(CONTENTS) is not bool:
                            CACHED_CONTENTS = CONTENTS
                            INPUT = f'Bucket Contents => {profile_name}'
                            CHOICE = runMenu(CACHED_CONTENTS, INPUT)
                        break
        else:
            CACHED_CONTENTS = []
            for PROFILE in CONFIG.PROFILES:
                if PROFILE in ignore:
                    continue
                if bucket in CONFIG.get_metadata_aws('cached_buckets', PROFILE, oci_region):
                    OSS = get_OSSclient(PROFILE, oci_region)
                    CONTENTS, TOTAL, LASTMOD = OSS.show_bucket_content(bucket, PROFILE)
                    if type(CONTENTS) is not bool:
                        CACHED_CONTENTS = CONTENTS
                        INPUT = f'Bucket Contents => {profile_name}'
                        CHOICE = runMenu(CACHED_CONTENTS, INPUT)
                    break
    else:
        if bucket is None:
            _show('buckets', oci_region, profile_name)
        else:
            _show('bucket_files', oci_region, profile_name, bucket)

def _show(target, oci_region, profile_name, bucket=None):
    try:
        if target == 'buckets':
            OSS = get_OSSclient(profile_name, oci_region, auto_refresh=False, cache_only=True)
            OSS.show_cache('cached_buckets', profile_name, oci_region)
        if target == 'bucket_files':
            OSS = get_OSSclient(profile_name, oci_region)
            CONTENTS, TOTAL, LASTMOD = OSS.show_bucket_content(bucket, profile_name)
            if type(CONTENTS) is not bool:
                print(json.dumps(CONTENTS, indent=4, sort_keys=True))
                Log.info(f"bucket: {bucket}, total size: {OSS.calculate_object_size(TOTAL)}, and last modified on: {str(LASTMOD)}")
            else:
                Log.critical(f"credentials not available to access OSS bucket {bucket} permission denied, or bucket does not exist")
        return True
    except:
        return False

def get_OSSclient(profile_name, region='us-ashburn-1', auto_refresh=False, cache_only=False):
    CLIENT = OSSclient(profile_name, region, cache_only)
    if auto_refresh:
        CLIENT.auto_refresh(profile_name)
    return CLIENT

def runMenu(DATA, INPUT):
    COUNT = 0
    FINAL = []
    TITLE = f'OSS Bucket Menu: {INPUT}'
    for data in DATA:
        COUNT = COUNT + 1
        RESULTS = []
        RESULTS.append(data)
        FINAL.append(RESULTS)
    SUBTITLE = f'showing {COUNT} available object(s)'
    JOINER = '\t\t'
    FINAL_MENU = Menu(FINAL, TITLE, JOINER, SUBTITLE)
    CHOICE = FINAL_MENU.display()
    return CHOICE

def get_region(ctx, profile_name):
    OCI_REGION = ctx.obj['REGION']
    if not OCI_REGION:
        OSS = get_OSSclient(profile_name)
        OCI_REGION = OSS.get_region_from_profile(profile_name)
    return OCI_REGION



import datetime, time, os, json, re, oci
from toolbox.logger import Log
from toolbox.misc import detect_environment
from configstore.configstore import Config
from . import iam_nongc
from .oci_config import OCIconfig
from tabulate import tabulate
from pathlib import Path
from tqdm.auto import tqdm
from oci.object_storage.models import CreateBucketDetails
try:
    import importlib_resources as resources
except:
    from importlib import resources

CONFIGSTORE = Config('ocitools')

class OSSclient():

    def __init__(self, profile_name, region='us-ashburn-1', cache_only=False):
        CONFIGSTORE = Config('ocitools')
        self.CONFIG = OCIconfig()
        self.CONFIGSTORE = Config('ocitools')
        self.size_table={0: 'Bs', 1: 'KBs', 2: 'MBs', 3: 'GBs', 4: 'TBs', 5: 'PBs', 6: 'EBs'}
        self.CACHE_UPDATE_INTERVAL = 60*60*24*7 # update cache every week
        self.OCI_PROFILE = profile_name
        self.OCI_REGION = region
        self.CACHE_ONLY = cache_only

        # get these from oci.config based on profile name and validate the config
        self.config = oci.config.from_file("~/.oci/config", profile_name)
        self.OCID = self.tenant = self.config.get("tenancy")
        self.user = self.config.get("user")
        self.fingerprint = self.config.get("fingerprint")
        self.keyfile = self.config.get("key_file")
        self.CONFIG_FROM_FILE = {
                'tenancy': self.tenant,
                'user': self.user,
                'fingerprint': self.fingerprint,
                'key_file': self.keyfile,
                'region': self.OCI_REGION
        }
        try:
            oci.config.validate_config(self.CONFIG_FROM_FILE)
        except:
            Log.critical('unable to setup a proper oci configuration file')

        if not self.CACHE_ONLY:
            self.get_connections()

    def get_connections(self):
        self.OSS = oci.object_storage.ObjectStorageClient(self.CONFIG_FROM_FILE)
        self.CLIENT = oci.identity.IdentityClient(self.CONFIG_FROM_FILE)
        self.NAMESPACE = self.OSS.get_namespace(compartment_id=self.OCID).data
        self.TENANTNAME = self.CLIENT.get_tenancy(tenancy_id=self.OCID).data.name
        return self.OSS, self.CLIENT, self.NAMESPACE, self.TENANTNAME

    def get_region_from_profile(self, profile_name):
        REGION = self.CONFIG.get_from_config('config', 'region', profile_name=profile_name)
        if REGION is None:
            Log.critical("Please run goat oci iam authenticate for the target profile before using the oss module")
        return REGION

    def mkdir_p(self, path):
        try:
            os.makedirs(path)
        except OSError as exc:  # Python >2.5
            pass

    def auto_refresh(self, profile_name='DEFAULT'):
        TIME_NOW = datetime.datetime.now().timestamp()
        try:    
            BUCKETS_TS = self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_buckets']['last_cache_update']
        except KeyError:
            BUCKETS_TS = 0    
        if TIME_NOW - float(BUCKETS_TS) > self.CACHE_UPDATE_INTERVAL:
            self.cache_buckets(profile_name)
        self.CONFIGSTORE = Config('ocitools')

    def refresh(self, profile_name='DEFAULT'):
        self.cache_buckets(profile_name)

    def oss_to_local(self, bucket_name, local_folder, filename, namespace):
        GET_OBJ = self.OSS.get_object(namespace, bucket_name, filename)
        TOTAL = int(GET_OBJ.headers.get('content-length', 0))
        OSS_PATH, OSS_FILENAME = self.get_oss_path_filename(filename)
        BASE = os.path.basename(filename)
        if filename is not None and BASE == OSS_FILENAME:
            LOCAL_FOLDER_PATH = os.path.join(*[local_folder, OSS_PATH])
            LOCAL_FULLPATH = os.path.join(LOCAL_FOLDER_PATH, OSS_FILENAME)
            self.mkdir_p(LOCAL_FOLDER_PATH)
        os.makedirs((local_folder), exist_ok=True)
        print()
        try:
            with open(LOCAL_FULLPATH, 'wb', buffering = 2 ** 24) as f, tqdm(
                desc=str(filename),
                total=TOTAL,
                unit='iB',
                unit_scale=True,
                unit_divisor=1024,
            ) as bar:
                for DATA in GET_OBJ.data.iter_content(chunk_size=1024):
                    SIZE = f.write(DATA)
                    bar.update(SIZE)
            print()
            return True
        except KeyboardInterrupt:
            print(color.RED + "\nCaught KeyboardInterrupt, exiting...\n" + color.RESET)
            try:
                sys.exit(0)
            except SystemExit:
                os._exit(0)
        return False

    def local_to_oss(self, bucket_name, local_path, namespace):
        CURRENT_TIME = datetime.datetime.now()
        object_name = local_path
        if not os.path.isfile(object_name) and not os.path.isdir(object_name):
            Log.critical("unable to upload object to OSS because the file does not exist")
        if os.path.isfile(object_name):
            with open(object_name, 'rb') as f:
                try:
                    obj = self.OSS.put_object(namespace, bucket_name, object_name, f)
                    Log.info(f"finished uploading {object_name} to {bucket_name}")
                    return True
                except:
                    Log.warn(f"problem uploading {object_name} to {bucket_name}")
                    return False
        else:
            dir = local_path + os.path.sep + "*"
            proc_list = []
            for f in glob(dir):
                if os.path.isfile(f):
                    p = Process(target=upload_to_object_storage, args=(self.CONFIG_FROM_FILE, namespace, bucket_name, f, profile_name))
                    p.start()
                    proc_list.append(p)
            for job in proc_list:
                job.join()
        return 0

    def upload_to_object_storage(self, config, namespace, bucket_name, local_path, profile_name):
        CURRENT_TIME = datetime.datetime.now()
        if os.path.isdir(local_path):
            for ROOT, DIRS, FILES in os.walk(local_path):
                for NAME in FILES:
                    FILE_PATH = os.path.join(ROOT, NAME)
                    with open(FILE_PATH, "rb") as in_file:
                        KEY = NAME
                        try:
                            self.OSS.put_object(namespace,
                                                bucket_name,
                                                KEY,
                                                in_file)
                            Log.info(f"finished uploading {NAME} to {bucket_name}")
                            return True
                        except:
                            Log.warn(f"problem uploading {KEY} to {bucket_name}")
                            return False
                            pass
        else:
            try:
                KEY = local_path
                with open(local_path, "rb") as in_file:
                    try:
                        self.OSS.put_object(namespace,
                                            bucket_name,
                                            KEY,
                                            in_file)
                        Log.info(f"finished uploading {NAME} to {bucket_name}")
                        return True
                    except:
                        Log.warn(f"problem uploading {KEY} to {bucket_name}")
                        return False
            except:
                pass

    def get_compartments(self):
        COMPARTMENTS = self.CLIENT.list_compartments(self.OCID, compartment_id_in_subtree=True).data
        ROOT_COMPARTMENT = self.CLIENT.get_compartment(compartment_id=self.OCID).data
        COMPARTMENTS.append(ROOT_COMPARTMENT)
        return COMPARTMENTS
       
    def show_bucket_content(self, bucket_name, profile_name, total=0):
        BUCKETS = CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_buckets'][self.OCI_REGION]
        for BUCKETNAME in BUCKETS:
            if BUCKETNAME == bucket_name:
                namespace = BUCKETS[BUCKETNAME]['namespace']
        TIMEOUT = time.time() + 90
        ITEMS = []
        LASTMOD = ''
        TOTAL = total
        if self.CACHE_ONLY:
            self.get_connections()
        prefix_files_name = ''
        BUCKETS = self.OSS.list_buckets(compartment_id=self.OCID, namespace_name=namespace).data
        OBJECTS = self.OSS.list_objects(namespace, bucket_name, prefix=prefix_files_name, fields="name")
        OBJECTS_LIST = OBJECTS.data.objects
        prefix_files_name = ''
        for NAME in OBJECTS_LIST:
            ITEMS.append(NAME.name)
        return ITEMS

    def cache_buckets(self, profile_name='DEFAULT'):
        osfields = ('namespaceName','bucketName','compartmentName','compartmentOcid')
        TIMESTAMP = str(datetime.datetime.now().timestamp())
        if profile_name not in self.CONFIGSTORE.PROFILES:
            self.CONFIGSTORE.create_profile(profile_name)
        
        Log.info('caching OCI object storage buckets...')
        BUCKETS_CACHE = {}
        COMPARTMENTS = self.get_compartments()
        for COMPARTMENT in COMPARTMENTS:
            BUCKETS = []
            NAMESPACE = []
            try:
                NAMESPACE = self.OSS.get_namespace(compartment_id=COMPARTMENT.id).data
                BUCKETS = self.OSS.list_buckets(compartment_id=COMPARTMENT.id, namespace_name=NAMESPACE).data
                COMP_NAME = str(COMPARTMENT.name)
                COMP_OCID = str(COMPARTMENT.id)
                if len(BUCKETS) > 0:
                    Log.debug("compartment name: " + COMP_NAME + " holds object storage buckets in region " + self.OCI_REGION)
                    for BUCKET in BUCKETS:
                        if BUCKET:
                            BUCKETNAME = str(BUCKET.name)
                            OBJECT_LIST = self.OSS.list_objects(NAMESPACE, BUCKETNAME, fields="name,timeCreated,size")
                            BUCKETS_CACHE[BUCKETNAME] = {
                                'name': BUCKETNAME,
                                'namespace': NAMESPACE,
                                'compartment': COMP_NAME,
                                'ocid': COMP_OCID
                            }
            except oci.exceptions.ServiceError as e:
                BUCKETS = ''
                pass

        BUCKETS_CACHE['last_cache_update'] = TIMESTAMP
        DICT = {}
        DICT[self.OCI_REGION] = BUCKETS_CACHE
        self.CONFIGSTORE.update_metadata(DICT, 'cached_buckets', profile_name, True)

    def get_cache(self, type, profile_name, subtype=None):
        if profile_name not in self.CONFIGSTORE.PROFILES:
            return None
        if type not in self.CONFIGSTORE.PROFILES[profile_name]['metadata']:
            return None
        if subtype is None:
            return self.CONFIGSTORE.PROFILES[profile_name]['metadata'][type][oci_region]
        else:
            if subtype in self.CONFIGSTORE.PROFILES[profile_name]['metadata'][type]:
                return self.CONFIGSTORE.PROFILES[profile_name]['metadata'][type][subtype]
            else:
                return None

    def show_cache(self, type, profile_name, oci_region):
        DATA = []
        for ENTRY in self.CONFIGSTORE.PROFILES[profile_name]['metadata'][type][oci_region]:
            DATA_ENTRY = {}
            if ENTRY != 'last_cache_update' and ENTRY != 'lastmod' and ENTRY != 'total':
                DATA_ENTRY['Name'] = ENTRY
                DATA.append(DATA_ENTRY)
        Log.info(f"{type}\n{tabulate(DATA, headers='keys', tablefmt='rst')}\n")

    def get_oss_path_filename(self, key):
        key = str(key)
        return key.replace(key.split('/')[-1],""), key.split('/')[-1]

    def delete_bucket(self, namespace, bucket_name, profile_name):
        try:
            RESPONSE = oci.pagination.list_call_get_all_results(self.OSS.list_preauthenticated_requests, namespace_name=namespace, bucket_name=bucket_name)
        except:
            Log.critical(f"FAIL: bucket {bucket_name} not found in profile: {profile_name}")
            return False

        if RESPONSE:
            for auth in RESPONSE.data:
                self.OSS.ObjectStorageClient.delete_preauthenticated_request(self.OSS, namespace_name=namespace, bucket_name=bucket_name, par_id=auth.id)
        try:
            self.OSS.delete_bucket(namespace, bucket_name)
            Log.info('INFO: deleted bucket from: ' + profile_name)
            return True
        except:
            cmd='oci --profile %s os object bulk-delete --bucket-name %s --force >/dev/null 2>&1' %(profile_name, bucket_name)
            os.system(cmd)
            self.OSS.delete_bucket(namespace, bucket_name)
            Log.info('deleted bucket from: ' + profile_name)
            return True
        return 0

    def create_bucket(self, ocid, namespace, bucket_name, profile_name):
        request = CreateBucketDetails(public_access_type='NoPublicAccess')
        request.compartment_id = ocid
        request.name = bucket_name
        bucket = self.OSS.create_bucket(namespace, request)
        if bucket.data.etag:
            Log.info("bucket created for " + profile_name + " and bucket tag " + bucket.data.etag)
            return True

def setup_config_file(profile_name, oci_region):
    CONFIG_FROM_FILE = {
        'tenancy': CONFIGSTORE.get_metadata('tenancy', profile_name),
        'user': CONFIGSTORE.get_metadata('user', profile_name),
        'fingerprint': CONFIGSTORE.get_metadata('fingerprint', profile_name),
        'key_file': CONFIGSTORE.get_metadata('key_file', profile_name),
        'region': oci_region
    }
    try:
        oci.config.validate_config(CONFIG_FROM_FILE)
    except:
        Log.critical('unable to setup a proper oci configuration file')
    return CONFIG_FROM_FILE    



from toolbox.logger import Log
from toolbox.misc import detect_environment
from configstore.configstore import Config
from .oci_config import OCIconfig
from . import iam_nongc
import datetime
from tabulate import tabulate
import os, oci, json
from . import shapes

CONFIGSTORE = Config('ocitools')

class SECRETclient():

    def __init__(self, profile_name, region='us-ashburn-1', cache_only=False):
        self.CONFIGSTORE = Config('ocitools')
        self.CONFIG = OCIconfig()
        self.CACHE_UPDATE_INTERVAL = 60*60*24*7 # update cache every week
        self.OCI_PROFILE = profile_name
        self.OCI_REGION = region
        self.CACHE_ONLY = cache_only

        # get these from oci.config based on profile name and validate the config
        self.config = oci.config.from_file("~/.oci/config", profile_name)
        self.OCID = self.tenant = self.config.get("tenancy")
        self.user = self.config.get("user")
        self.fingerprint = self.config.get("fingerprint")
        self.keyfile = self.config.get("key_file")
        self.CONFIG_FROM_FILE = {
                'tenancy': self.tenant,
                'user': self.user,
                'fingerprint': self.fingerprint,
                'key_file': self.keyfile,
                'region': self.OCI_REGION
        }
        try:
            oci.config.validate_config(self.CONFIG_FROM_FILE)
        except:
            Log.critical('unable to setup a proper oci configuration file')

        if not self.CACHE_ONLY:
            self.get_connections()

    def get_connections(self):
        #Log.info(f"connecting to OCI as {self.OCI_PROFILE} via {self.OCI_REGION}...")
        self.CLIENT = oci.identity.IdentityClient(self.CONFIG_FROM_FILE)
        self.TENANTNAME = self.CLIENT.get_tenancy(tenancy_id=self.OCID).data.name
        self.VAULT = oci.vault.VaultsClient(self.CONFIG_FROM_FILE)
        self.SECRETS = oci.secrets.SecretsClient(self.CONFIG_FROM_FILE)
        self.KEYS = oci.key_management.KmsVaultClient(self.CONFIG_FROM_FILE)
        self.COMPOSITE = oci.key_management.KmsVaultClientCompositeOperations(self.KEYS)
        return self.CLIENT, self.TENANTNAME, self.VAULT, self.SECRETS, self.KEYS, self.COMPOSITE

    def get_secrets_cache(self, profile_name='DEFAULT'):
        FIELDS = ['id','key_id','lifecycle_state','secret_name','description']
        SECRETS_CACHE = {}
        if profile_name not in self.CONFIGSTORE.PROFILES:
            self.CONFIGSTORE.create_profile(profile_name)
        TIMESTAMP = str(datetime.datetime.now().timestamp())
        self.get_connections()
        Log.info("caching secrets across all compartments...")
        COMPARTMENTS = self.get_compartments()
        for COMPARTMENT in COMPARTMENTS:
            if COMPARTMENT.lifecycle_state != 'ACTIVE':
                continue
            SECRETS = []
            VAULT_ID = ''
            try:
                SECRETS = self.VAULT.list_secrets(compartment_id=COMPARTMENT.id, lifecycle_state="ACTIVE").data
                COMP_NAME = str(COMPARTMENT.name)
                COMP_OCID = str(COMPARTMENT.id)
                if len(SECRETS) > 0:
                    Log.debug("compartment name: " + COMP_NAME + " holds secrets in region " + self.OCI_REGION)
                    for DATA in SECRETS:
                        if DATA:
                            ID = DATA.id
                            KEY_ID = DATA.key_id
                            SECRET_RESPONSE = self.SECRETS.get_secret_bundle(secret_id=ID).data
                            TYPE = SECRET_RESPONSE.secret_bundle_content.content_type
                            CONTENT = SECRET_RESPONSE.secret_bundle_content.content
                            SECRET_NAME = DATA.secret_name
                            DESC = DATA.description
                            NEW_VAULT_ID = DATA.vault_id
                            STATE = DATA.lifecycle_state
                            SECRETS_CACHE[ID] = {
                                'secret_name': SECRET_NAME,
                                'content': CONTENT,
                                'type': TYPE,
                                'description': DESC,
                                'compartment': COMP_NAME,
                                'lifecycle_state': STATE
                            }
            except oci.exceptions.ServiceError as e:
                SECRETS = ''
                pass

        SECRETS_CACHE['last_cache_update'] = TIMESTAMP
        DICT = {}
        DICT[self.OCI_REGION] = SECRETS_CACHE
        self.CONFIGSTORE.update_metadata(DICT, 'cached_secrets', profile_name, True)

    def describe(self, secret_ocid, profile_name):
        try:
            self.get_connections()
        except:
            return None
        SECRET_RESPONSE = self.SECRETS.get_secret_bundle(secret_id=secret_ocid).data
        TYPE = SECRET_RESPONSE.secret_bundle_content.content_type
        CONTENT = SECRET_RESPONSE.secret_bundle_content.content
        DATA = self.VAULT.get_secret(secret_id=secret_ocid).data
        DICT = {}
        DICT[secret_ocid] = DATA
        DICT['content'] = CONTENT
        return DICT

    def get_cached_secrets(self, profile_name):
        if profile_name in self.CONFIGSTORE.PROFILES:
            if 'cached_secrets' in self.CONFIGSTORE.PROFILES[profile_name]['metadata']:
                return self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_secrets'][self.OCI_REGION]
        else:
            return None

    def refresh(self, type, profile_name):
        if type == 'cached_secrets':
            self.get_secrets_cache(profile_name)
        self.CONFIGSTORE = Config('ocitools')

    def get_compartments(self):
        COMPARTMENTS = self.CLIENT.list_compartments(self.OCID, compartment_id_in_subtree=True).data
        ROOT_COMPARTMENT = self.CLIENT.get_compartment(compartment_id=self.OCID).data
        COMPARTMENTS.append(ROOT_COMPARTMENT)
        return COMPARTMENTS

    def delete_secret(self, ocid):
        DETAILS = oci.vault.models.ScheduleSecretDeletionDetails()
        RESPONSE = self.VAULT.schedule_secret_deletion(ocid, DETAILS)
        return RESPONSE

    def list_secrets(self, ocid):
        self.get_connections()
        SECRETS = self.VAULT.list_secrets(compartment_id=ocid, lifecycle_state="ACTIVE").data
        return SECRETS

    def create_secret(self, compartment_id, secret_content, secret_name, vault_id, key_id, secret_description):
        self.get_connections()
        VAULT_COMPOSITE = oci.vault.VaultsClientCompositeOperations(self.VAULT)
        SECRET_CONTENT_DETAILS = oci.vault.models.Base64SecretContentDetails(content_type=oci.vault.models.SecretContentDetails.CONTENT_TYPE_BASE64, name=secret_name, stage="CURRENT", content=secret_content)
        SECRETS_DETAILS = oci.vault.models.CreateSecretDetails(compartment_id=compartment_id, description=secret_description, secret_content=SECRET_CONTENT_DETAILS, secret_name=secret_name, vault_id=vault_id, key_id=key_id)
        RESPONSE = VAULT_COMPOSITE.create_secret_and_wait_for_state(create_secret_details=SECRETS_DETAILS, wait_for_states=[oci.vault.models.Secret.LIFECYCLE_STATE_ACTIVE])
        return RESPONSE.data

    def create_key(self, key_name, compartment_id, service_endpoint):
        vault_management_client = oci.key_management.KmsManagementClient(self.CONFIG_FROM_FILE, service_endpoint=service_endpoint)
        vault_management_client_composite = oci.key_management.KmsManagementClientCompositeOperations(vault_management_client)
        KEY_SHAPE = oci.key_management.models.KeyShape(algorithm="AES", length=32)
        KEY_DETAILS = oci.key_management.models.CreateKeyDetails(compartment_id=compartment_id, display_name=key_name, key_shape=KEY_SHAPE)
        RESPONSE = vault_management_client_composite.create_key_and_wait_for_state(KEY_DETAILS, wait_for_states=[oci.key_management.models.Key.LIFECYCLE_STATE_ENABLED])
        return RESPONSE.data

    def schedule_deletion_key(self, deletion_time, key_id, service_endpoint):
        vault_management_client = oci.key_management.KmsManagementClient(self.CONFIG_FROM_FILE, service_endpoint=service_endpoint)
        vault_management_client_composite = oci.key_management.KmsManagementClientCompositeOperations(vault_management_client)
        SCHEDULE_KEY_DELETION_DETAILS = oci.key_management.models.sc(time_of_deletion=deletion_time)
        RESPONSE = vault_management_client_composite.schedule_key_deletion_and_wait_for_state(schedule_key_deletion_details, wait_for_states=[oci.key_management.models.Key.LIFECYCLE_STATE_PENDING_DELETION])
        return RESPONSE

    def auto_refresh(self, profile_name='DEFAULT'):
        TIME_NOW = datetime.datetime.now().timestamp()
        try:
            SECRET_TS = self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_secrets']['last_cache_update']
        except KeyError:
            SECRET_TS = 0
        if TIME_NOW - float(SECRET_TS) > self.CACHE_UPDATE_INTERVAL:
            self.get_secrets_cache(profile_name)
        self.CONFIGSTORE = Config('ocitools')



import click, os, gnureadline, csv
from toolbox.logger import Log
from .computeclient import MyComputeClient
from toolbox.menumaker import Menu
from tabulate import tabulate
from configstore.configstore import Config
from toolbox.misc import set_terminal_width
from .iam import get_latest_profile

CONFIG = Config('ocitools')

try: input = raw_input
except NameError: raw_input = input

@click.group('compute', invoke_without_command=True, help='compute to manage the instances and related configuration', context_settings={'help_option_names':['-h','--help'], 'max_content_width': set_terminal_width()})
@click.option('-m', '--menu', help='use the menu to perform compute actions', is_flag=True, show_default=True, default=False, required=False)
@click.pass_context
def compute(ctx, menu):
    if menu:
        ctx.obj['MENU'] = True
    else:
        ctx.obj['MENU'] = False

@compute.command(help='manually refresh compute cached data', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
@click.argument('cache_type', required=False, default='all', type=click.Choice(['all', 'instances', 'public_ips', 'regions', 'auto']))
def refresh(ctx, cache_type):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    _refresh(cache_type, profile_name, oci_region)

def _refresh(cache_type, profile_name, oci_region):
    try:    
        COMPUTE = get_compute_client(profile_name, oci_region, auto_refresh=False, cache_only=True)
        if cache_type == 'all':
            COMPUTE.refresh(profile_name)
        if cache_type == 'instances':
            COMPUTE.cache_instances(profile_name)
        if cache_type == 'public_ips':
            COMPUTE.cache_public_ips(profile_name)
        if cache_type == 'regions':
            COMPUTE.cache_regions(profile_name)
        if cache_type == 'auto':
            COMPUTE.auto_refresh(profile_name)
        return True
    except:
        return False

@compute.command(help='show the data stored in compute cache (instacnes, IPs, regions)', context_settings={'help_option_names':['-h','--help']})
@click.pass_context
@click.argument('target', required=False, default='all', type=click.Choice(['all', 'instances', 'public_ips', 'regions', 'orphaned_ebs', 'orphaned_snaps']))
def show(ctx, target):
    profile_name = ctx.obj['PROFILE']
    oci_region = get_region(ctx, profile_name)
    if ctx.obj["MENU"] and target == "instances":
        CACHED_INSTANCES = {}
        DATA = []
        for PROFILE in CONFIG.PROFILES:
            if PROFILE == 'latest':
                continue
            if PROFILE == profile_name:
                COMPUTE = get_compute_client(profile_name, oci_region, auto_refresh=False, cache_only=True)
                RES = COMPUTE.show_cache(profile_name, oci_region, 'cached_instances', display=False)
                for data in RES:
                    NAME = data["display_name"].ljust(50)
                    IP = data["private_ips"].strip()
                    STATE = data["lifecycle_state"]
                    details = NAME + "\t" + IP + "\t" + STATE
                    DATA.append(details)
                break
        INPUT = 'Login Manager'
        CHOICE = runMenu(DATA, INPUT)
        if CHOICE:
            IP = CHOICE[0].split("\t")[1]
            Log.info(f"you chose to SSH to {IP} which will be implemented later...")
    else:
        _show(target, profile_name, oci_region, display=True)

def _show(target, profile_name, oci_region, display):
    try:
        COMPUTE = get_compute_client(profile_name, oci_region, auto_refresh=False, cache_only=True)
        if target == 'instances' or target == 'all':
            COMPUTE.show_cache(profile_name, oci_region, 'cached_instances', display)
        if target == 'public_ips' or target == 'all':
            COMPUTE.show_cache(profile_name, oci_region, 'cached_public_ips', display)
        if target == 'regions' or target == 'all':
            COMPUTE.show_cache(profile_name, oci_region, 'cached_regions', display)
        return True
    except:
        return False

def get_compute_client(profile_name, region='us-ashburn-1', auto_refresh=False, cache_only=False):
    CLIENT = MyComputeClient(profile_name, region, cache_only)
    if auto_refresh:
        CLIENT.auto_refresh(profile_name)
    return CLIENT

def runMenu(DATA, INPUT):
    COUNT = 0
    FINAL = []
    TITLE = f'Compute Menu: {INPUT}'
    for data in DATA:
        COUNT = COUNT + 1
        RESULTS = []
        RESULTS.append(data)
        FINAL.append(RESULTS)
    SUBTITLE = f'showing {COUNT} available object(s)'
    JOINER = '\t\t'
    FINAL_MENU = Menu(FINAL, TITLE, JOINER, SUBTITLE)
    CHOICE = FINAL_MENU.display()
    return CHOICE

def get_region(ctx, profile_name):
    oci_region = ctx.obj['REGION']
    if not oci_region:
        COMPUTE = get_compute_client(profile_name)
        oci_region = COMPUTE.get_region_from_profile(profile_name)
    return oci_region



from toolbox.logger import Log
from toolbox.misc import detect_environment
from configstore.configstore import Config
from .oci_config import OCIconfig
from . import iam_nongc
import datetime
from tabulate import tabulate
import os, oci, json
from . import shapes

CONFIGSTORE = Config('ocitools')

class MyComputeClient():

    def __init__(self, profile_name, region='us-ashburn-1', cache_only=False):
        self.CONFIGSTORE = Config('ocitools')
        self.CONFIG = OCIconfig()
        self.OCI_REGION = region
        self.CACHE_UPDATE_INTERVAL = 60*60*24*7 # update cache every week
        self.OCI_PROFILE = profile_name
        self.CACHE_ONLY = cache_only
        # get these from oci.config based on profile name and validate the config
        self.config = oci.config.from_file("~/.oci/config", profile_name)
        self.OCID = self.tenant = self.config.get("tenancy")
        self.user = self.config.get("user")
        self.fingerprint = self.config.get("fingerprint")
        self.keyfile = self.config.get("key_file")
        self.CONFIG_FROM_FILE = {
                'tenancy': self.tenant,
                'user': self.user,
                'fingerprint': self.fingerprint,
                'key_file': self.keyfile,
                'region': self.OCI_REGION
        }

        try:
            oci.config.validate_config(self.CONFIG_FROM_FILE)
        except:
            Log.critical('unable to setup a proper oci configuration file')

        if not self.CACHE_ONLY:
            self.get_connections()

    def get_connections(self):
        self.CLIENT = oci.identity.IdentityClient(self.CONFIG_FROM_FILE)
        self.TENANTNAME = self.CLIENT.get_tenancy(tenancy_id=self.OCID).data.name
        self.COMPUTE = oci.core.ComputeClient(self.CONFIG_FROM_FILE)
        self.NETWORK = oci.core.VirtualNetworkClient(self.CONFIG_FROM_FILE)
        return self.CLIENT, self.COMPUTE, self.NETWORK, self.TENANTNAME

    def auto_refresh(self, profile_name='DEFAULT'):
        TIME_NOW = datetime.datetime.now().timestamp()
        try:
            INSTANCES_TS = self.CONFIGSTORE.PROFILES[profile_name]['metadata']['cached_instances']['last_cache_update']
        except KeyError:
            INSTANCES_TS = 0
        if TIME_NOW - float(INSTANCES_TS) > self.CACHE_UPDATE_INTERVAL:
            self.cache_instances(profile_name)
        self.CONFIGSTORE = Config('ocitools')

    def refresh(self, profile_name='DEFAULT'):
        self.cache_instances(profile_name)

    def DisplayInstances(self, instances, compartmentName, instancetype, region_name):

        TIMESTAMP = str(datetime.datetime.now().timestamp())
        AllPredefinedTags = True 
        NoValueString = "n/a"
        FieldSeperator = ","
        EndLine = "\n"
        customertags = []
        header = "NAME,STATE,SERVICE,SHAPE,COMPARTMENT,AD".replace(",", FieldSeperator)
        COMPUTE_CACHE = {}

        for instance in instances:
            privateips = ""
            publicips = ""
            instancetypename = ""
            tagtxt = ""
            OS = ""
            if instancetype == "Compute":
                OCPU, MEM, SSD = shapes.ComputeShape(instance.shape)
                response = self.COMPUTE.list_vnic_attachments(compartment_id = instance.compartment_id, instance_id = instance.id)
                vnics = response.data
                try:
                    for vnic in vnics:
                        responsenic = self.NETWORK.get_vnic(vnic_id=vnic.vnic_id)
                        nicinfo = responsenic.data
                        privateips = privateips + nicinfo.private_ip + " "
                        publicips = publicips + nicinfo.public_ip + " "
                except:
                    privateips = NoValueString
                    publicips = NoValueString
                instancetypename = "Compute"
                namespaces = instance.defined_tags
                shape = instance.shape
                # Get OS Details
                try:
                    response = self.COMPUTE.get_image(instance.source_details.image_id)
                    imagedetails = response.data
                    OS = imagedetails.display_name
                except:
                    OS = NoValueString
                prefix, AD = instance.availability_domain.split(":")
            try:
                namespaces = instance.defined_tags
                for customertag in customertags:
                    try:
                        tagtxt = tagtxt + FieldSeperator + namespaces[customertag[0]][customertag[1]]
                    except:
                        tagtxt = tagtxt + FieldSeperator + NoValueString
            except:
                tagtxt = ""  # No Tags

            COMPUTE_CACHE[instance.display_name] = {
                                'display_name': instance.display_name,
                                'lifecycle_state': instance.lifecycle_state,
                                'instance_type': instancetypename,
                                'shape': shape,
                                'compartment_name': compartmentName,
                                'public_ips': publicips,
                                'private_ips': privateips,
                                'AD': AD
            }
            COMPUTE_CACHE['last_cache_update'] = TIMESTAMP
        return COMPUTE_CACHE

    def cache_instances(self, profile_name='DEFAULT'):
        if profile_name not in self.CONFIGSTORE.PROFILES:
            self.CONFIGSTORE.create_profile(profile_name)
        TIMESTAMP = str(datetime.datetime.now().timestamp())
        self.get_connections()
        Log.info("caching compute instances...")
        response = oci.pagination.list_call_get_all_results(self.CLIENT.list_compartments,self.OCID,compartment_id_in_subtree=True)
        compartments = response.data
        RootCompartment = oci.identity.models.Compartment()
        RootCompartment.id = self.OCID
        RootCompartment.name = "root"
        RootCompartment.lifecycle_state = "ACTIVE"
        compartments.insert(0, RootCompartment)

        for compartment in compartments:
            compartmentName = compartment.name
            if compartment.lifecycle_state == "ACTIVE":
                Log.debug("processing compartment:  " + compartmentName)
                compartmentID = compartment.id
                try:
                    response = oci.pagination.list_call_get_all_results(self.COMPUTE.list_instances,compartment_id=compartmentID)
                    if len(response.data) > 0:
                        COMPUTE_CACHE = self.DisplayInstances(response.data, compartmentName, "Compute", self.OCI_REGION)
                        DATADICT = {}
                        DATA = []
                        for I in COMPUTE_CACHE:
                            if 'last_cache_update' not in I:
                                DATA.append(COMPUTE_CACHE[I])
                        if DATA:
                            DATADICT = DATA
                        DICT = {}
                        DICT[self.OCI_REGION] = COMPUTE_CACHE 
                        self.CONFIGSTORE.update_metadata(DICT, 'cached_instances', profile_name, True)
                        self.CONFIGSTORE = Config('ocitools')
                except:
                    pass

    def show_cache(self, profile_name, oci_region, type, display):
        DATA = []
        for ENTRY in self.CONFIGSTORE.PROFILES[profile_name]['metadata'][type][oci_region]:
            DATA_ENTRY = {}
            if ENTRY != 'last_cache_update':
                DATA_ENTRY['Name/ID'] = ENTRY
                for PROPERTY in self.CONFIGSTORE.PROFILES[profile_name]['metadata'][type][oci_region][ENTRY]:
                    DATA_ENTRY[PROPERTY] = self.CONFIGSTORE.PROFILES[profile_name]['metadata'][type][oci_region][ENTRY][PROPERTY]
                DATA.append(DATA_ENTRY)
        if DATA != []:
            if display is True:
                Log.info(f"{type}\n{tabulate(DATA, headers='keys', tablefmt='rst')}\n")
            return DATA

    def show_all_cache(self, oci_region, profile_name):
        self.show_cache(profile_name, oci_region, 'cached_instances', display=False)
        self.show_cache(profile_name, oci_region, 'cached_public_ips', display=False)
        self.show_cache(profile_name, oci_region, 'cached_regions', display=False)

    def get_region_from_profile(self, profile_name):
        REGION = self.CONFIG.get_from_config('config', 'region', profile_name=profile_name)
        if REGION is None:
            Log.critical("Please run goat oci iam authenticate for the target profile before using the oss module")
        return REGION



#!/usr/bin/env python3

import os, sys, oci, csv, json
base = '/opt/native/gen2cli'

if len(sys.argv) != 3:
    print("INFO: all parameters are required to run this tool:")
    print("- 1st param = tenantOcid")
    print("- 2nd param = region")
    sys.exit()

# init the items we need
tenantOcid = sys.argv[1]
region_name = sys.argv[2]

storage = []

# capture all regions
osfields = ('namespaceName','bucketName','compartmentName','compartmentOcid')
config = oci.config.from_file(profile_name='default')
id_client = oci.identity.IdentityClient(config)
tenantName = id_client.get_tenancy(tenancy_id=tenantOcid).data.name
os_client = oci.object_storage.ObjectStorageClient(config)
filename = base + "/logs/%s_all_compartments_%s.csv" %(tenantOcid, region_name)
osfilename = base + "/logs/%s_all_storage_%s.csv" %(tenantOcid, region_name)

print("INFO: refreshing storage buckets for region: " + region_name)
with open(osfilename, 'w', newline='', encoding='utf-8') as oscsvfile:
    compartments = id_client.list_compartments(tenantOcid, compartment_id_in_subtree=True).data
    root_compartment = id_client.get_compartment(compartment_id=tenantOcid).data
    compartments.append(root_compartment)

    oswriter = csv.writer(oscsvfile, lineterminator='\n')
    oswriter.writerow(osfields)

    for compartment in compartments:
        try:
            namespace = os_client.get_namespace(compartment_id=compartment.id).data
            buckets = os_client.list_buckets(compartment_id=compartment.id, namespace_name=namespace).data
            comp_name = str(compartment.name)
            comp_ocid = str(compartment.id)
            if len(buckets) > 0:
                print("INFO: compartment name: " + comp_name + " holds object storage buckets in region " + region_name)
                for bucket in buckets:
                    if bucket:
                        bucketname = str(bucket.name)
                        oscsvfile.write("{},{},{},{}\n".format(namespace, bucketname, comp_name, comp_ocid))
        except oci.exceptions.ServiceError as e:
            buckets = ''
            pass

f = open(osfilename, 'r')

f = open(osfilename, 'r')
reader = csv.DictReader(f)
for each in reader:
    row = {}
    for field in osfields:
        row[field] = each[field]
        if row not in storage:
            storage.append(row)

if len(storage) > 0:
    home = os.getenv("HOME")
    fname = home + '/.gen2/storage/' + tenantName + '/' + region_name + '.json'
    os.makedirs(os.path.dirname(fname), exist_ok=True)
    with open(fname, "w") as outfile:
        json.dump(storage, outfile, indent=2, sort_keys=True)



# Oracle OCI - Instance report script - Shaped definition
# Version: 1.4 29-November 2018
# Written by: richard@oc-blog.com
#

def ComputeShape(name):
    # returns OCPU count, Memory in GB and Local Storage in TB
    if name == "VM.Standard1.1":
        return 1, 7, 0
    elif name == "VM.Standard.E2.1":
        return 1, 8, 0
    elif name == "VM.Standard2.1":
        return 1, 15, 0
    elif name == "VM.Standard1.2":
        return 2, 14, 0
    elif name == "VM.Standard2.2":
        return 2, 30, 0
    elif name == "VM.Standard.E2.2":
        return 2, 16, 0
    elif name == "VM.Standard1.4":
        return 4, 28, 0
    elif name == "VM.Standard2.4":
        return 4, 60, 0
    elif name == "VM.Standard.E2.4":
        return 4, 32, 0
    elif name == "VM.Standard1.8":
        return 8, 56, 0
    elif name == "VM.Standard2.8":
        return 8, 120, 0
    elif name == "VM.Standard.E2.8":
        return 8, 64, 0
    elif name == "VM.Standard1.16":
        return 16, 112, 0
    elif name == "VM.Standard2.16":
        return 16, 240, 0
    elif name == "VM.Standard2.24":
        return 24, 320, 0
    elif name == "VM.DenseIO1.4":
        return 4, 60, 3.2
    elif name == "VM.DenseIO1.8":
        return 8, 120, 6.4
    elif name == "VM.DenseIO2.8":
        return 8, 120, 6.4
    elif name == "VM.DenseIO1.16":
        return 16, 240, 12.8
    elif name == "VM.DenseIO2.16":
        return 16, 240, 12.8
    elif name == "VM.DenseIO2.24":
        return 24, 320, 25.6
    elif name == "VM.GPU2.1":
        return 12, 104, 0
    elif name == "BM.GPU2.2":
        return 28, 192, 0
    elif name == "VM.CPU3.1":
        return 6, 90, 0
    elif name == "VM.CPU3.2":
        return 12, 180, 0
    elif name == "VM.CPU3.4":
        return 24, 360, 0
    elif name == "BM.CPU3.8":
        return 52, 768, 0
    elif name == "BM.Standard1.36":
        return 36, 256, 0
    elif name == "BM.Standard2.52":
        return 52, 768, 0
    elif name == "BM.StandardE2.64":
        return 64, 512, 0
    elif name == "BM.HighIO1.36":
        return 36, 512, 12.8
    elif name == "BM.DenseIO1.36":
        return 36, 512, 28.8
    elif name == "BM.DenseIO2.52":
        return 52, 768, 51.2
    elif name == "BM.HPC2.36":
        return 36, 384, 0
    elif name == "BM.RACLocalStorage1.72":
        return 72, 1024, 64
    elif name == "Exadata.Quarter1.84":
        return 84, 1440, 84
    elif name == "Exadata.Half1.168":
        return 168, 2880, 168
    elif name == "Exadata.Full1.336":
        return 336, 5760, 336
    elif name == "Exadata.Quarter2.92":
        return 92, 1440, 106
    elif name == "Exadata.Half2.184":
        return 184, 2880, 212
    elif name == "Exadata.Full12.368":
        return 368, 5760, 424

    else:
        return 0,0,0

